/*
 cs_files.h
  MyCSeries include for file management
   much of this is not implemented!

  Version 0.4

  Created: December 31, 2005
  Revised: August 1, 2006


    Version 1.0 has not yet been reached, so if you have a name conflict,
  please feel free to contact me via E-mail with a list of alternative
  names, so the name conflict can be resolved by the next release.
*/

/*
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 *  This source code is offered for use in the public domain.  You may use,
 * modify or distribute it freely.
 *
 *  This code is distributed in the hope that it will be useful but WITHOUT
 * ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY DISCLAIMED.
 *  This includes but is not limited to warranties of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.
 */

#ifndef __CS_FILES__
# define __CS_FILES__

# include <cs_system.h>
# include <stdio.h>

# if PRAGMA_ONCE
#  pragma once
# endif

# ifdef __cplusplus
   extern "C" {
# endif

# if PRAGMA_IMPORT
#  pragma import on
# endif

# if PRAGMA_STRUCT_ALIGN
#  pragma options align=mac68k
# elif PRAGMA_STRUCT_PACKPUSH
#  pragma pack(push,2)
# elif PRAGMA_STRUCT_PACK
#  pragma pack(2)
# endif

 /* constants */

  enum
  {
    /* error codes */
     kFS_volume_not_found       = -100, /* volume index, ID or in path couldn't be found */
     kFS_directory_not_found    = -101, /* directory index, ID or in path couldn't be found */
     kFS_file_not_found         = -102, /* directory index, ID or in path couldn't be found */
     kFS_object_conflict        = -103, /* expected a directory, file or volume and got something else */
     kFS_volume_offline         = -104, /* volume has been taken off-line */
     kFS_duplicate_name         = -105, /* desired object name equal to another in the same directory */
     kFS_reserved_name          = -106, /* desired object name is reserved by the system */
     kFS_missing_name           = -107, /* operation requires name (non-positive length or NULL pointer) */
     kFS_name_too_long          = -108, /* name length is too long either in directory or in general */
     kFS_drive_not_found        = -109, /* asked for drive/volume reference for a non-existant drive number */

     kFS_volume_full            = -110, /* not enough free space on volume */
     kFS_directory_full         = -111, /* can't add another object because the directory is full */
     kFS_file_full              = -112, /* file has reached it's logical limit */
     kFS_disk_full              = -113, /* no space for a new partition (not to be confused with "volume full") */

     kFS_volume_locked          = -115, /* volume is locked or read-only/write-protected */
     kFS_directory_locked       = -116, /* directory is locked or read-only/write-protected */
     kFS_file_locked            = -117, /* file is locked or read-only/write-protected */
     kFS_disk_write_protected   = -118, /* disk (covers all volumes) is write protected or read-only */
     kFS_file_permissions       = -119, /* file permissions error (couldn't get permission?) */

     kFS_fork_creation_failure  = -120, /* could not allocate a file fork (or directory) */
     kFS_fork_conflict          = -121, /* both native and non-native fork are present */
     kFS_fork_busy              = -122, /* could not push out all forks because one or more are in use */
     kFS_move_into_offspring    = -123, /* attempted to move a directory into one of it's own */
     kFS_deletion_of_open_file  = -124, /* attempted to delete a file that's in use */
     kFS_wfile_already_open     = -125, /* file with write permission (or any perm) is being opened again */
     kFS_wfile_permission       = -126, /* write permission error */
     kFS_file_reference_error   = -127, /* file reference error */
     kFS_bad_file_reference     = -128, /* invalid file reference error */
     kFS_reference_overflow     = -129, /* too many files or references being used at once */

     kFS_root_corrupt           = -130, /* bad root or master directory block(s) */
     kFS_internal_error         = -131, /* file system internal error */
     kFS_data_lost              = -132, /* file system lost some data - backup your stuff */
     kFS_external_access        = -133, /* external file system - can't do anything */
     kFS_hardware_io_error      = -134, /* platform system hardware I/O error (not good) */
     kFS_memory_or_slot_error   = -135, /* platform file system read or load failure due to limitations */
     kFS_volume_already_mounted = -136, /* attempt to mount an already mounted volume */
     kFS_unusable_signature     = -137, /* system software doesn't like the sig bytes on a partition */
     kFS_op_not_supported       = -138, /* system software will not allow that operation for this disk/volume */

     kFS_file_eof_error         = -140, /* file is being positioned or read past it's logical end */
     kFS_negative_position      = -141, /* file is being positioned below the bottom of the file */
     kFS_get_position_error     = -142  /* an error occured getting the current position of the file */
  };

  enum
  { /* fs_create flags */
     _create_non_native_forks   = 0x00000001  /* force non-native fork creation */
  };

  enum
  { /* fs_open_fork flags */
     _open_write_permission     = 0x00000001, /* otherwise you may not be able to write to the fork */
     _open_resolve_aliases      = 0x00000080  /* follow all aliases/links/shortcuts if it is one of them */
  };

/* ------- structures */

 /*
  About forks
   Fork names are eight characters in ASCII (eight bytes).  Characters beyond
     the first eight bytes are ignored.
   The data fork is always the main file.

   For the MacOS file system (and other HFS related file systems):
     The resource fork is built-in.
     Other fork names work natively on HFS+ under MacOS 9 or later.
   If forks cannot be managed natively, then a directory named "XXXXXXXX.FRK"
     (Xs represent the fork name) will be created for them, containing a file
     with a name equal to the main file (representing the data fork).
   It is recommended that fork names be all-caps alpha-numeric.

   Forks are created automatically but may be created manually.
     There are no error numbers regarding fork creation with duplicate names.
   The only error regarding with duplicate fork names is when there's a
     conflict between a native fork and the directory for fork allocation
     holding a file with the same name (native and non-native existing file
     fork at the same time).  In order to access the fork, only one can exist
     at the same time (you may have to manually remove one).
   If a fork allocating directory already exists then that directory will be
     used in managing the file fork regardless of whether the fork can exist
     natively.
 */

  typedef struct cs_filespec/*(20+)*/
  {
     SInt32             encoding; /* points to the encodings in cs_text_encodings.h */
     SInt32             directory; /* platform specific */
     SInt16             volume; /* platform specific */
     volatile SInt16    length; /* number of characters in the object's name */
     void               *name; /* user text pointer - may be NULL */
     char               *fork; /* user C-string pointer - NULL/empty counts as data fork */
  } cs_filespec;

/* ------- typedef */

# if TARGET_OS_MAC
#  ifndef __FILES__
#   include <Files.h>
#  endif
   typedef FSSpec       platform_filespec;
# else
   typedef char         platform_filespec[288]; /* just a C-string */
# endif

/* ------- globals */

/* name of the non-native resource fork directory name (excluding ".FRK"/"_FRK") */
extern char         cs_resource_fork_name[256];

/* ------- prototypes */

 /* file deletion is deliberately not supplied for security reasons */

 /* uses platform string lists recognized by cs_strings */
  EXTERN_API_C(int) fs_locate_file_by_strlist(
     cs_filespec *outFile, /* can be NULL */
     short strID,
     int index,
     int range, /* count of strings in string list strID */
     short pathsStrID); /* string list containing all of the search paths to it */

  EXTERN_API_C(int) build_platform_filespec(
    cs_filespec *inFile,
    platform_filespec *outFile);

 /* create a file and/or a fork if the name is specified;
   cross-platform programs should use the suffix in the file name or read
   the heading in the file to determine the file type rather than relying
   on the file and creator types
 */
  EXTERN_API_C(int) fs_create(
    cs_filespec *desiredFile,
    UInt32 flags,
    UInt32 type, /* platform specific */
    UInt32 creator); /* platform specific */

 /* makes all of the file's forks non-native for compatibility;
   all forks except for the data fork must not be in use or else the
   "fork is busy" error code will be returned */
  EXTERN_API_C(int)
   fs_push_out_forks(cs_filespec *inFile);

 /* returns the new file reference; a negative result means an error code */
  EXTERN_API_C(int)
   fs_open_fork(cs_filespec *srcFile, UInt32 flags);

  EXTERN_API_C(int)
   fs_close_fork(int reference);

 /* negative offsets will be interpreted as an offset to the end of the fork */
  EXTERN_API_C(platform_handle)
   fs_read_to_handle(int refNum, SInt64 offset, SInt32 length);

# if PRAGMA_STRUCT_ALIGN
#  pragma options align=reset
# elif PRAGMA_STRUCT_PACKPUSH
#  pragma pack(pop)
# elif PRAGMA_STRUCT_PACK
#  pragma pack()
# endif

# ifdef PRAGMA_IMPORT_OFF
#  pragma import off
# elif PRAGMA_IMPORT
#  pragma import reset
# endif

# ifdef __cplusplus
   }
# endif

#endif /*!defined(__CS_FILES__)*/
