/*
 cs_math.h
  MyCSeries include for useful math routines

  Version 0.4

  Created: December 1, 2005
  Revised: December 28, 2006


    Version 1.0 has not yet been reached.  If you have a name conflict,
  please feel free to contact me via E-mail with a list of alternative
  names, so the name conflict can be resolved by the next release.
*/

/*
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 *  This source code is offered for use in the public domain.  You may use,
 * modify or distribute it freely.
 *
 *  This code is distributed in the hope that it will be useful but WITHOUT
 * ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY DISCLAIMED.
 *  This includes but is not limited to warranties of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.
 */

#ifndef __CS_MATH__
# define __CS_MATH__

# include <cs_system.h>
# include <math.h>

# if PRAGMA_ONCE
#  pragma once
# endif

# ifdef __cplusplus
   extern "C" {
# endif

# if PRAGMA_IMPORT
#  pragma import on
# endif

/* typedefs */

# ifdef env68k
#  define _cs_eval_double           Float80 /* for compatibility in this include */
#  define _cs_eval_ldouble          Float96
   extern const _cs_eval_ldouble    long_double_one;
# else
#  define _cs_eval_double           native_double_t
#  define _cs_eval_ldouble          long double
# endif

/* constants */

/* these macros are also stored in the const globals below */
#define kPi                 3.1415926535897932384626433832795028841971694L
#define kRadicalTwo         1.4142135623730950488016887242096980785696719L
#define kTwelfthRootTwo     1.0594630943592952645618252949463417007792043L
#define kLogBaseTwoTen      3.3219280948873623478703194294893901758648314L
#define kGoldenRatio        1.6180339887498948482045868343656381177203092L

 extern const _cs_eval_double       double_t_one, double_t_pi;
 extern const _cs_eval_double       double_t_radical_two;
 extern const _cs_eval_double       double_t_twelfth_root_two;
 extern const _cs_eval_double       double_t_log2_10;
 extern const _cs_eval_double       double_t_golden_ratio;
/* higher precision (where available) */
 extern const _cs_eval_ldouble      long_double_pi;
 extern const _cs_eval_ldouble      long_double_radical_two;
 extern const _cs_eval_ldouble      long_double_twelfth_root_two;
 extern const _cs_eval_ldouble      long_double_log2_10;
 extern const _cs_eval_ldouble      long_double_golden_ratio;

/* prototypes */

# if TARGET_CPU_68K
   UInt32 __fsqrtu32(UInt32 n);
   UInt32 __fsqrtu64(UInt64 n);
   UInt32 __sqrtu32(UInt32 n);
   UInt32 __sqrtu64(UInt64 n);
#  if 0 /*TARGET_RT_MAC_68881*/
#   define isqrt                    __fsqrtu32
#   define isqrt64                  __fsqrtu64
#  else
#   define isqrt                    __sqrtu32
#   define isqrt64                  __sqrtu64

#   ifndef log10
#    define log10(x)                (log2(x)/double_t_log2_10)
#   endif
#  endif
# else
   UInt32 isqrt(UInt32 n);
   UInt32 isqrt64(UInt64 n);
# endif

# ifdef THINK_C
 /* either defined by the compiler, the compiler's public object code
   (fp.h or math.h with MathLib or SANE on MacOS), or by iocustom.c */
   EXTERN_API_C( _cs_eval_double )
     asin( _cs_eval_double x );
   EXTERN_API_C( _cs_eval_double )
     acos( _cs_eval_double x );
# else
 /* `power' should be used instead of `pow' */
#  define power(x,y)                pow(x,y)
# endif

 /* defined by iocustom.c */
  EXTERN_API_C( long double )
    cot( long double x );

  EXTERN_API_C( UInt32 )
    _isqrt64( UInt32, UInt32 );

  EXTERN_API_C( ufixed )
    FixedSqrt( ufixed );

# ifndef VFix16Mul
   EXTERN_API_C( sfixed )
     VFix16Mul( sfixed, sfixed );
# endif
# ifndef VFix16Div
   EXTERN_API_C( sfixed )
     VFix16Div( sfixed, sfixed );
# endif

 /* returns a list of all of the prime factors in `n'; terminated by a zero

   NOTE: the result points to an internal array - don't "free" the result
 */
  EXTERN_API_C( UInt32* )
    cs_prime_list(UInt64 n);

# ifdef PRAGMA_IMPORT_OFF
#  pragma import off
# elif PRAGMA_IMPORT
#  pragma import reset
# endif

# ifdef __cplusplus
   }
# endif
#endif /* defined(__CS_MATH__) */
