/*
 cs_system.h

  Version 0.4

  Created: November 25, 2004
  Revised: June 16, 2007


    Version 1.0 has not yet been reached.  If you have a name conflict,
  please feel free to contact me via E-mail with a list of alternative
  names, so the name conflict can be resolved by the next release.
*/

/*
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 *  This source code is offered for use in the public domain.  You may use,
 * modify or distribute it freely.
 *
 *  This code is distributed in the hope that it will be useful but WITHOUT
 * ANY WARRANTY.  ALL WARRANTIES, EXPRESSED OR IMPLIED ARE HEREBY DISCLAIMED.
 *  This includes but is not limited to warranties of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.
 */

#ifndef __CS_SYSTEM__
# define __CS_SYSTEM__

# include <cseries.h>
# include <cs_system_errors.h>

# if PRAGMA_ONCE
#  pragma once
# endif

# ifdef __cplusplus
   extern "C" {
# endif

# if PRAGMA_IMPORT
#  pragma import on
# endif

 /* was VoidRoutineUPP */
  typedef CALLBACK_API_C(void,VoidRoutineProc)(void);

/* constants */

  enum
  { /* platform types - translate_platform_error uses these */
     _platform_MacOS,   /* MacOS 7.1-8.1 (Classic); Apple Computer, Inc. */
     _platform_MacOSX,  /* MacOS 10.2 compatible; Apple Computer, Inc. */
     _platform_Unix,    /* Unix - hp(?) */
     _platform_Linux,   /* GNU compatible linux */
   /* 4 */
     _platform_Win32,   /* Microsoft Windows (R) (32 bit) */
     _platform_Win64,   /* Microsoft Windows (R) Vista (64 bit) */
     _platform_OS2,     /* OS/2 */
     _platform_MSDOS,   /* MS-DOS 5 compatible */
   /* 8 */
     _platform_Cygwin,  /* cygwin */
     _platform_FreeBSD, /* FreeBSD */
     _platform_NetBSD,  /* NetBSD */
     _platform_OpenBSD, /* OpenBSD */
   /* 12 */
     _platform_BeOS,    /* BeOS */
     _platform_HPUX,    /* HP-UX */
     _platform_Solaris, /* Sun Solaris */
     _platform_Irix,    /* SGI Irix */
   /* 16 */
     _platform_Aix      /* IBM Aix */
  };

  enum
  { /* exit code disable flags */
     kDFOutOfMemory             = 0x1,      /* application heap is full */
     kDFAccessFault             = 0x2,      /* unmapped or private address */
     kDFIllegal                 = 0x4,      /* bad instruction */
     kDFBadAddress              = 0x8,      /* branch/jump to odd address */
     kDFCoreFailure             = 0x10,     /* internal failure or low-level s/w error */
     kDFStackEnteredHeap        = 0x20,     /* stack is now in the heap */
     kDFZoneFreeNegative        = 0x40,     /* the "free space" value went negative. */
     kDFBadIntegerMath          = 0x80,     /* divide by zero (usually) */
     kDFFPException             = 0x100,    /* floating-point exception */
     kDFBadFPUOpcode            = 0x200,    /* the FPU (or emulator) can't handle it */
     kDFTrap                    = 0x400,    /* trap or debugger error */
     kDFPatchTrap               = 0x800,    /* bad attempt with patch */
     kDFPriviledgeViolation     = 0x1000,   /* instruction priviledged violation */
     kDFCodeReadFailure         = 0x2000,   /* no code loaded */
     kDFDataReadFailure         = 0x4000,
     kDFLaunchFailure           = 0x8000,   /* we have (or something) can't launch */
     kDFPurged                  = 0x10000,  /* code or data was purged */
     kDFNoInterrupt             = 0x40000,  /* no interrupt installed */
     kDFNoFileSystem            = 0x100000, /* file system map is no longer present */
     kDFIOFailure               = 0x200000, /* hardware bus error - not an access fault */
     kDFInvalid                 = 0x800000  /* won't call (same result to using "atexit") */
  };

/* ------- structures */

  enum
  {
    /* cs_init_params flags - CSI Miami is better than CSI New York? */
     kCSI_approximate_ms        = 0x1, /* wastes 3 secs refining "cs_ms_per_second" using the clock seconds */
     kCSI_inhibit_atexittoshell = 0x2, /* calling "atexittoshell" does nothing */

     kCSI_uses_SDL_timer        = 0x80 /* should be set if SDL is being used */
  };

  typedef struct cs_init_params
  {
     /* your passed pointer only goes for `init_params_size' bytes 
       (including the `init_params_size' field) */
     SInt32         init_params_size;

     UInt32         flags;

     /* initial master pointer count; because of the risk of memory fragmentation,
       it's better to have too many master pointers than too little

       (master pointers are what link platform_handles)
     */
     SInt32         init_master_count;

     /* will expand... */
  } cs_init_params;

/* ------- typedefs */

# if TARGET_OS_MAC || defined(__MACTYPES__)
#  define platform_ptr                  Ptr
#  define platform_handle               Handle
# else
#  define platform_ptr                  char*
#  define platform_handle               char**
# endif

/* ------- globals */

 /* "cs_ms_per_second" is a global variable that is adjustable (in case the
   hardware - emulated or not - is inaccurate).  It is not meant for any other
   purpose -- changing this value on platforms where milliseconds are native
   may do nothing at all as far as the value returned by machine_milliseconds.

    "cs_ms_second_sync" is the number of milliseconds in difference from the
   clock seconds -- unreliable on systems with innacurate or asynchronous
   timing.
 */
  extern UInt32       cs_ms_per_second, cs_ms_second_sync;

/* ------- prototypes */

 /* You MUST call "cs_initialize" before using "atexittoshell" or any of the
   "cs_" routines */

  EXTERN_API_C(int)
   cs_initialize( const cs_init_params *parameters );

 /* be sure to call this at least once every hour to make sure that the value
   returned by "machine_milliseconds" doesn't suffer from 16 or 32 bit wrapping
   (no carrying to higher bound) - on some systems, you may have to call this
   every half a second.

  "flags" is yet to be defined - pass zero for now
 */
  EXTERN_API_C(int)
   cs_runtime( UInt32 flags );

 /* translates the error code of the passed platform to a CSeries kxx_ error */
  EXTERN_API_C(int)
   translate_platform_error(int inErr, int inPlatform);

 /* heap management/emulation */

 /* creates a new platform-specific non-relocatable block
   returns NULL on failure
 */
  EXTERN_API_C(platform_ptr)
   new_platform_ptr( long );

 /* disposes the platform-specific non-relocatable block; returns error code */
  EXTERN_API_C(int)
   dispose_platform_ptr( platform_ptr );

 /* returns the size of the platform-specific non-relocatable block; may also
   return a negative value (representing an error code)
 */
  EXTERN_API_C(long)
   get_platform_ptr_size( platform_ptr );

 /* sets the size of the platform-specific non-relocatable block;
   returns an error code on failure

    for blocks that may increase in size, it's recommended to use "handles"
   rather than "pointers"
 */
  EXTERN_API_C(int)
   set_platform_ptr_size( platform_ptr, long newSize );

 /* creates a new platform-specific relocatable block; returns NULL on failure

   pass non-zero for `lock' to lock it immediately;
    otherwise it will be unlocked when returned
 */
  EXTERN_API_C(platform_handle)
   new_platform_handle( long, boolean lock );

 /* disposes the platform-specific relocatable block; returns error code */
  EXTERN_API_C(int)
   dispose_platform_handle( platform_handle );

 /* returns the size of the platform-specific relocatable block; may also
   return a negative value (representing an error code)
 */
  EXTERN_API_C(long)
   get_platform_handle_size( platform_handle );

 /* sets the size of the platform-specific relocatable block;
   returns an error code
 */
  EXTERN_API_C(int)
   set_platform_handle_size( platform_handle, long newSize );

 /* adds to the size of the platform-specific relocatable block;
   returns an error code
 */
  EXTERN_API_C(int)
   append_platform_handle( platform_handle, long addedSize );

 /* returns the lock and purge flags of the platform-specific relocatable
   block; returns an error code
 */
  EXTERN_API_C(int)
   get_platform_handle_state( platform_handle, boolean *lock, boolean *purge );

 /* sets the lock and purge flags of the platform-specific relocatable block;
   returns an error code
 */
  EXTERN_API_C(int)
   set_platform_handle_state( platform_handle, boolean lock, boolean purge );

 /* fixes the platform-specific relocatable block in place; the system and
   MyCSeries will not move it when it is locked; returns error code */
  EXTERN_API_C(int)
   lock_platform_handle( platform_handle );

 /* unlocks the platform-specific relocatable block in place; the system and
   MyCSeries may move it any point afterwards; returns error code */
  EXTERN_API_C(int)
   unlock_platform_handle( platform_handle );

 /* moves the platform-specific relocatable block up the heap and away from
   the lower section likely to contain non-relocateable blocks */
  EXTERN_API_C(int)
   move_platform_handle_high( platform_handle );

 /* timing */

 /*
   machine_milliseconds

   sets the destination operand to the number of milliseconds since machine
    startup -- or since SDL_Init if using SDL timing

   the native header (such as macintosh_cseries.h) must define this with
    a more native prototype and/or inline
 */
 /*EXTERN_API_C(void) machine_milliseconds( UInt64& );*/

 /* wait_for_click_or_keypress waits for the mouse button to be clicked or
   a non-modifier key to be pressed with a maximum amount of time;
   returns 1 if it was interrupted, 0 if not; an error code otherwise

   the maximum value for "maxMilliseconds" is about 49.71 days
 */
  EXTERN_API_C(int)
   wait_for_click_or_keypress( UInt32 maxMilliseconds, boolean waitUp );

 /* the program's main bottleneck for idling */
  EXTERN_API_C(void) global_idle_proc( void );

 /* for disabling the screen saver (use revive_screen_saver to re-enable it
   via atexit) */
  EXTERN_API_C(void) kill_screen_saver( void );
  EXTERN_API_C(void) revive_screen_saver( void );

 /* the program handles these */
  EXTERN_API_C(void) stop_interface_fade( void );

# ifdef THINK_C
#  include <stdlib.h>

#  define atexit(f)                     atexittoshell(f,-1L)
   extern int cs_exit(int);
#  define exit                          cs_exit
# elif defined(__SC__) && (defined(MPW_CPLUS) || defined(MPW_C))
#  include <RTLib.h>
# endif

/*
  "atexittoshell" is for code that is needed to be run before the program
    completely terminates (this includes exceptions)

  for an example of the disable flags:
   if you don't want it to be called if there's no memory available OR if the
    stack has entered the heap.
   You would pass (kDFOutOfMemory | kDFStackEnteredHeap).
*/
  EXTERN_API_C(int) atexittoshell(VoidRoutineProc, int disableFlags);
 /*
   if a routine wants to change it's disable flags as far as "atexittoshell"
 */
  EXTERN_API_C(int) setexitdisableflags(VoidRoutineProc, int newDisableFlags);

 /* debugging */

 /* if ";g;" is found in the string then the text before the semi-colon is
   logged to the debugging log; otherwise, the program enters the debugger */
  EXTERN_API_C(int) dprintf(const char*,...); /* returns string length */

  EXTERN_API_C(void) initialize_debugger(boolean sysOnly);
  EXTERN_API_C(void) close_debugging_log(void);

 /* `data' can contain NULLs */
  EXTERN_API_C(void) debugger_log(const void *data, int length);

# ifdef PRAGMA_IMPORT_OFF
#  pragma import off
# elif PRAGMA_IMPORT
#  pragma import reset
# endif

# ifdef __cplusplus
   }
# endif

#endif /* defined(__CS_SYSTEM__) */
