/*
 cs_system_errors.h
  MyCSeries cross-platform system error codes

  Revised: 2008.06.14 14:37 GMT

  (cs_system.h includes this file - read cs_system.h)
*/

/*
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 *  This source code is offered for use in the public domain.  You may use,
 * modify or distribute it freely.
 *
 *  This code is distributed in the hope that it will be useful but WITHOUT
 * ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY DISCLAIMED.
 *  This includes but is not limited to warranties of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.
 */

#ifndef __CS_SYSTEM_ERRORS_H
# define __CS_SYSTEM_ERRORS_H

/* system error codes */

  enum
  { /* CSeries errors */
     kCS_no_error                   = 0,    /* well, this actually seems to apply to about everything */
     kCS_miserable_failure          = -1,
     kCS_already_initialized        = -2,
     kCS_user_canceled              = -3,   /* hit cancel button, etc. */
     kCS_unrecognized_version       = -4,

     kCS_flag_suppressed            = 1,    /* returned by atexittoshell if it's disabled */

    /* platform system errors (CSeries universal codes) */
     kCS_index_not_found            = -5,   /* generic error */
     kCS_invalid_index              = -6,   /* generic error */
     kCS_unimplemented_error        = -7,   /* generic error */
     kCS_io_error                   = -8,   /* generic I/O opperation error */
     kCS_unknown_error              = -9,   /* unknown/invalid error code */

     kCS_memory_full_error          = -10,
     kCS_insufficient_stack         = -11,  /* not enough stack space for operation */
     kCS_read_only_heap_zone_err    = -12,  /* hardware or software error in a read-only zone */
     kCS_bad_address                = -13,  /* address is misaligned or out of range*/
     kCS_null_pointer               = -14,
     kCS_null_handle                = -15,
     kCS_block_verify_error         = -16,  /* generic heap block verify error */
     kCS_free_block_error           = -17,  /* tried to identify a block in a zone but it's free/redirecting block */
     kCS_purge_block_error          = -18,  /* tried to purge a block that is against being purged */
     kCS_move_block_error           = -19,  /* tried to move a block that is against being moved */

     kCS_debugger_not_installed     = -20,  /* platform debugger not installed */
     kCS_system_error               = -21,  /* general platform system error */
     kCS_system_io_sw_error         = -22,  /* platform system I/O software error */
     kCS_system_io_error            = -23,  /* platform system general I/O error */
     kCS_system_status_error        = -24,  /* platform system s/w or h/w error on reading status or polling */
     kCS_system_control_error       = -25,  /* platform system s/w or h/w error on changing status */
     kCS_system_access_error        = -26,  /* platform system device I/O access or reference error */
     kCS_system_read_error          = -27,  /* platform system device I/O read error */
     kCS_system_write_error         = -28,  /* platform system device I/O write error */
     kCS_system_aborted             = -29,  /* platform system aborted I/O call */

     kCS_driver_removal_error       = -30,  /* attempted to remove an open driver */
     kCS_driver_install_error       = -31,  /* couldn't install driver */
     kCS_driver_port_in_use         = -32,  /* driver openning I/O access on a port that is in use by another */
     kCS_driver_configuration       = -33,  /* hardware config problem */
     kCS_driver_not_open            = -34,  /* the necessary driver was not open during an I/O call */
     kCS_device_unit_out_of_bounds  = -35,  /* platform system has no more device unit entries */
     kCS_device_unit_reference      = -36,  /* bad device unit reference or unit is empty (insignificant) */

     kCS_io_verify_error            = -40,  /* general verify I/O error */
     kCS_initialize_io_error        = -41,  /* general I/O error - couldn't initialize */
     kCS_insufficient_io_run        = -42,  /* general I/O error - not enough to qualify */
     kCS_could_not_continue_run     = -43,  /* general I/O error - couldn't qualify for next step in run */
     kCS_read_compare_verify        = -44,  /* read-compare-verify I/O error */
     kCS_checksum_error             = -45,  /* checksum verification I/O error */
     kCS_marker_not_found           = -46,  /* marker wasn't found I/O error */
     kCS_sector_not_found           = -47,  /* sector wasn't found I/O error */
     kCS_track_not_found            = -48,  /* track wasn't found I/O error */
     kCS_unreachable_track          = -49,  /* drive is not built to access track/side */

     kCS_parameter_error            = -50,  /* user passed bad parameter(s) */
     kCS_data_too_long              = -51,  /* generic length check error */
     kCS_platform_software_error    = -52,  /* any platform-specific software error */

     kCS_system_font_error          = -60,  /* platform system font error */
     kCS_clipboard_no_data          = -61,  /* platform software is trying to get non-existant data from the clipboard */

     kCS_socket_number_error        = -70,  /* multi-cast address error */
     kCS_non_local_failure          = -71,  /* can't send non-locally */
     KCS_collisions_at_write        = -72,  /* too many collisions at write */
     KCS_protocol_attachment        = -73,  /* protocol attach/detach failure */
     kCS_serial_port_power          = -74,  /* serial port not powered */
     kCS_io_sync_error              = -75,  /* generic sync I/O error */

     kCS_BIOS_error                 = -80,  /* generic ROM BIOS error (where available) */
     kCS_bios_error                 = kCS_BIOS_error,
     kCS_PRAM_error                 = -81,  /* generic PRAM error (where available) */
     kCS_pram_error                 = kCS_PRAM_error,
     kCS_clock_read_error           = -82,  /* could not read clock (more than once?) */
     kCS_clock_write_error          = -83,  /* could not set/write to clock */

     kCS_SCSI_receiver_error        = -90,  /* SCSI parity/framing error */
     kCS_SCSI_received_break        = -91   /* SCSI received a "break" */
  };

#endif /*!defined(__CS_SYSTEM_ERRORS)*/
