/*
 macintosh_cseries.h
  MacOS runtime include for MyCSeries

  Version 0.4

  Created: March 12, 2004
  Revised: June 14, 2008

  Written by Adam P.

   Part of my reorganized (but limited) universal cseries library.
     Originally this header was part of (and all in two headers) my cseries
     substitution code for the cseries game software library.

   Version 1.0 has not yet been reached.  If you have a name conflict,
     please contact me via E-mail with a list of alternative names so the
     name conflict can be resolved by next release.

   The entire header code (excluding cseries.h) used to be in here,
     and now it's nearly empty, as the prototypes 'n enums 'n such
     were moved to their (so-called) respected "cs_xxxx" headers.
*/

/*
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 *  This source code is offered for use in the public domain.  You may use,
 * modify or distribute it freely.
 *
 *  This code is distributed in the hope that it will be useful but WITHOUT
 * ANY WARRANTY.  ALL WARRANTIES, EXPRESSED OR IMPLIED ARE HEREBY DISCLAIMED.
 *  This includes but is not limited to warranties of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.
 */

#ifndef __MACINTOSH_CSERIES_H
# define __MACINTOSH_CSERIES_H

# ifndef __MACTYPES__
#  include <MacTypes.h>
# endif

# ifndef __DATETIMEUTILS__
#  include <DateTimeUtils.h>
# endif

# ifndef __QDOFFSCREEN__
#  include <QDOffscreen.h>
# endif

# ifndef __FILES__
#  include <Files.h>
# endif

# ifndef __SCRIPT__
#  include <Script.h>
# endif

# ifndef __DEVICES__
#  include <Devices.h>
# endif

# ifndef __PROCESSES__
#  include <Processes.h>
# endif

# ifndef __TIMER__
#  include <Timer.h>
# endif

# if PRAGMA_ONCE
#  pragma once
# endif

# ifdef __cplusplus
   extern "C" {
# endif

/* MyCSeries may use any of the routines in these includes;
 the ANSI routines should be either supplied (or emulated) */
# include <stdlib.h>
# include <stdio.h>

/* the "cs_" files should include <cseries.h> */

# include "cs_dialogs.h"
# include "cs_graphics.h"
# include "cs_strings.h"
# include "cs_text.h"
# include "cs_system.h"
# include "cs_math.h"
# include "cs_files.h"

/*
  the original individual order is kept in the backup archives for historical
    reasons; for one, to show the order of what was needed to be added
    (or changed) to compile the Marathon 2 source code.
*/

/* ------- constants */

# define MACINTOSH_TICKS_PER_SECOND     60
# define SYSTEM_TICKS_PER_SECOND        MACINTOSH_TICKS_PER_SECOND

# define asyncUncompleted               1

  enum
  { /* function key codes */
     kcF1        = 0x7A,
     kcF2        = 0x78,
     kcF3        = 0x63,
     kcF4        = 0x76,
     kcF5        = 0x60,
     kcF6        = 0x61,
     kcF7        = 0x62,
     kcF8        = 0x64,
     kcF9        = 0x65,
     kcF10       = 0x6D,
     kcF11       = 0x67,
     kcF12       = 0x6F,
     kcF13       = 0x69,
     kcF14       = 0x6B,
     kcF15       = 0x71
  };

/* ------- macros and traps */

# ifdef env68k
#  pragma parameter __D0 get_a0()
   extern SInt32 get_a0(void) = 0x2008;
#  pragma parameter __D0 get_a1()
   extern SInt32 get_a1(void) = 0x2009;
#  pragma parameter __D0 get_a5()
   extern SInt32 get_a5(void) = 0x200D;

/* very dangerous... */
#  pragma parameter __D0 set_a5(__D1)
   extern SInt32 set_a5(SInt32) = {0x200D, 0x2A41};

/*
#  pragma parameter _machine_partseconds(__A1)
    -- Microseconds does not save and restore (D0-D2/A0/A1)!
*/
   extern pascal void _machine_partseconds( UInt64*, UInt32 ) =
    {0x4EB0, 0x81E1, 0x64C, 0x2208, 0x4C57, 0x1002, 0x4C5F, 0x402, 0x205F,
     0x20C1, 0x2080};
/*
    ; about 100+_Microseconds cycles (over 200 cycles)
    JSR         ([0x64C])   ; Microseconds
    MOVE.L      A0,D1
    TDIVU.L     (SP),D2:D1
    DIVU.L      (SP)+,D2:D0
    MOVEA.L     (SP)+,A0
    MOVE.L      D1,(A0)+
    MOVE.L      D0,(A0)
*/
#  define machine_milliseconds(x)       _machine_partseconds(&(x),cs_ms_per_second)
# else
#  ifdef __cplusplus
    inline void _machine_partseconds(UInt64 *x, UInt32 d)
    {
       struct UnsignedWide _tmp;

       Microseconds(&_tmp); *(x)=*(UInt64*)&_tmp; UDiv64U32(*(x),d);
    }
    inline void machine_milliseconds(UInt64 &x)
    {
       struct UnsignedWide _tmp;

       Microseconds(&_tmp); (x)=*(UInt64*)&_tmp; UDiv64U32((x),cs_ms_per_second);
    }
#  else
#   define _machine_partseconds(x,d)    do{struct UnsignedWide _tmp; \
     Microseconds(&_tmp); *(x)=*(UInt64*)&_tmp; UDiv64U32(*(x),d);}while(0)
#   define machine_milliseconds(x)      do{struct UnsignedWide _tmp; \
     Microseconds(&_tmp); (x)=*(UInt64*)&_tmp; UDiv64U32((x),cs_ms_per_second);}while(0)
#  endif
# endif

/* ------- prototypes */

# define hide_cursor()                  HideCursor()
# define show_cursor()                  InitCursor()
# define mouse_still_down()             StillDown()
# define get_system_tick_count()        TickCount()


//#ifdef envppc
//# define VFix16Mul(n,d) FixMul(n,d)
//# define VFix16Div(n,d) FixDiv(n,d)
//#endif

//# define set_dialog_cursor_tracking(x) SetDialogTracksCursor(dialog,x)

#if !TARGET_API_MAC_CARBON
  unsigned char *GetDCtlName(short refNum);
  SInt16 GetSlotFromGDevice(GDHandle gd);

  OSErr get_driver_name(UInt8 slot, Str63 str);
#endif

  OSErr get_my_fsspec(FSSpec*);

  int appendstrtomenu(MenuRef menu, short strs_id);

# ifdef __cplusplus
   }
# endif

#endif /* defined(__MACINTOSH_CSERIES_H) */
