/*
 iocustom.c
  sample file for required runtime routines

  For MyCSeries version 0.4

  Created: December 25, 2004
  Revised: June 24, 2007

  Written (mostly) by Adam P.

  This is a sample file for the remaining runtime routines required by MyCSeries.
  The appropriate MyCSeries library file should be linked with this when compiled.

  This will _not_ work with MacTraps and/or the original SANE -- read the README
    in "for Symantec C, THINK C" on how to replace them.
*/
/*
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 *  This source code is offered for use in the public domain.  You may use,
 * modify or distribute it freely.
 *
 *  This code is distributed in the hope that it will be useful but WITHOUT
 * ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY DISCLAIMED.
 *  This includes but is not limited to warranties of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.
 */

#include <cseries.h>
#include <cs_math.h>
#include <cs_strings.h>
#include <cs_system.h>
#include <cs_byte_swapping.h>

#include <SizeTDef.h>

#include <stdio.h>
#include <stdarg.h>

#include <ctype.h>
#include <errno.h>
#include <time.h>

/* constants */

/* the vsnprintf here has not been proven to work at all! */
#define VSNPRINTF_SUPPORTED             0

#ifndef VSNPRINTF_SUPPORTED
# if defined(__MRC__) || defined(__SC__) || defined(THINK_C)
#  define VSNPRINTF_SUPPORTED           1
# else
#  define VSNPRINTF_SUPPORTED           0
# endif
#endif

/*** define this as your application signature, such as the shell name ***/
#ifndef PROGRAM_SIGNATURE
# define PROGRAM_SIGNATURE              "<unnamed>"
#endif

static const char    _log_date[] = "-- %Y*%m*%d %H:%M:%S GMT\015";

/* globals */

#if defined(THINK_C) && TARGET_OS_MAC
# include <QuickDraw.h>
# ifdef env68k
   QDGlobals    qd;
# endif
#endif

 /* the debugging strings are no longer supported */

char            temporary[512]; /* 512 bytes to avoid overflow */

static const char    opening_str[] = "\015Opening log for " PROGRAM_SIGNATURE;
static const char    closing_str[] = "\015Closing log";

static boolean  has_logged = FALSE;

boolean         sys_debugger_available = FALSE, use_sys_debugger = FALSE;

/* -------- prototypes */

int snprintf(char *buf, long max, const char *form, ...);
int vsnprintf(char *buf, long max, const char *form, va_list args);

/* ------- debugging code */

#if TARGET_OS_MAC
# include <Folders.h>
# include <MacMemory.h>

#  undef CALL_NOT_IN_CARBON
#  define CALL_NOT_IN_CARBON            0
#  include <Debugging.h>

 /* taken from Debugging ('cause dprintf now conflicts in an irresolvable manner) */
# define LocalLMGetMacJmp()             (0+*((UInt32*)0x0120))
# define LocalLMGetMacJmpFlag()         (0+*((UInt8*)0x0BFF))

# define ISLOWLEVELDEBUGGERCALLABLE()   (LocalLMGetMacJmpFlag()!=(UInt8)-1 \
   && (LocalLMGetMacJmpFlag()&(UInt8)0xE0)==(UInt8)0x60 \
   && LocalLMGetMacJmp()!=0 && LocalLMGetMacJmp()!=(UInt32)-1)
#elif !defined(ISLOWLEVELDEBUGGERCALLABLE)
# define ISLOWLEVELDEBUGGERCALLABLE()   (FALSE)
#endif

#ifdef env68k
# pragma parameter __D0 Get68KInterruptLevel()
  extern short Get68KInterruptLevel(void) = {0x40C0,0xE048,0x240,7};
# undef ATTASKLEVEL0
# define ATTASKLEVEL0()                 (Get68KInterruptLevel()==0)
#elif envppc
# ifndef ATTASKLEVEL0
#  define ATTASKLEVEL0()                (TaskLevel()==0)
# endif
#else
# define ATTASKLEVEL0()                 (TRUE) /* ?? */
#endif

#undef exit
 void exit(int);

/* debug messaging temporary */
static UInt8        _dbgTemp[VSNPRINTF_SUPPORTED ? 264:512];

static FILE         *debugging_log = (FILE*)NULL;

static char         debugging_log_namestring[] = "debugging_log";

/*** it is recommended that the user add their own system debugger support here ***/

/* * *
Brief Terminology

  The "debugger log" is an alternative to debugger interruptions, and is intended to
be seperate from stdout/stderr.  It covers all of the debugger routines in cseries.h
and cs_system.h except for dprintf's that don't end in ";g;".
* * */

void initialize_debugger(boolean sysOnly)
{
   close_debugging_log(); /* just in case */

   sys_debugger_available = ISLOWLEVELDEBUGGERCALLABLE() ? TRUE:FALSE;

   if (sysOnly)
   {
      use_sys_debugger = sys_debugger_available;
   } else if (debugging_log!=(FILE*)NULL)
   {
      time_t        t;

      /* if it doesn't exist, make it exist */
      if ((debugging_log=fopen(debugging_log_namestring,"r+"))!=(FILE*)NULL)
         has_logged = TRUE;
      else if ((debugging_log=fopen(debugging_log_namestring,"wa"))==(FILE*)NULL)
         return;

      /* append the "opening" string and the date */
      fseek(debugging_log,0,SEEK_END);
      ftell(debugging_log);
      fputs(opening_str,debugging_log);
      time(&t);
      strftime((char*)_dbgTemp,100,_log_date,gmtime(&t));
      fputs((char*)_dbgTemp,debugging_log);
   }
}

void close_debugging_log()
{
   time_t       t;

   if (debugging_log==NULL)
      return;

   fputs(closing_str,debugging_log);
   time(&t);
   strftime((char*)_dbgTemp,100,_log_date,gmtime(&t));
   fputs((char*)_dbgTemp,debugging_log);
   fclose(debugging_log);
   debugging_log = NULL;

#if REMOVE_EMPTY_LOGFILES
   if (!has_logged)
      remove(debugging_log_namestring);
#endif
}

void debugger_log(const void *s, int z)
{
   char         buf[102];
   time_t       t;

   if (debugging_log==NULL)
   {
      return;
   }
   time(&t);
   strftime(&buf[1],100,_log_date,gmtime(&t));
   buf[0] = '\015';
   fputs(buf,debugging_log);
   fwrite(s,1,z,debugging_log);
}

void _debugMsg(const char *str, const char *file, int line, int level)
{
   if (str==(const char*)NULL || str==(const char*)-1)
      str = (const char*)empty;

   if (level>=0)
      psprintf(_dbgTemp,"file \"%s\", line #%d: %s",file,line,str);
   else
      psprintf(_dbgTemp,"file \"%s\", halt at line #%d: %s",file,line,str);

   if (sys_debugger_available) // && DEBUG
   {
#if TARGET_OS_MAC
      if (level==0) /* not pausing; add ';g' to go immediately after printing */
         *(UInt16*)&dbgTemp[(dbgTemp[0]=MIN(dbgTemp[0],253)+2)-2] = ';g';
      DebugStr(_dbgTemp);
#endif
   } else
      debugger_log(&_dbgTemp[1],_dbgTemp[0]);

   if (level<0)
   {
      exit(1);
#if TARGET_OS_MAC
      DebugStr("\pFailed to exit.  You must manually terminate.");
#endif
   }
}

int dprintf(const char *form, ...)
{
   va_list      args;
   char         *s;
   int          len;

   va_start(args,form);
#if VSNPRINTF_SUPPORTED
   len = vsnprintf((char*)&_dbgTemp[1],255,form,args);
#else
   if ((len=vsprintf((char*)&_dbgTemp[1],form,args))>255)
      len = 255;
#endif
   va_end(args);

   s = memchr(&_dbgTemp[1],';',_dbgTemp[0]=len);

   if (debugging_log==NULL || !ATTASKLEVEL0() || s==NULL || s[1]!='g' || s[2]!=';')
   {
      if (sys_debugger_available)
      {
#if TARGET_OS_MAC
         DebugStr(_dbgTemp);
#else
         /* write for your own compiler software library */
#endif
      } else
      {
         /* otherwise, we are screwed */
      }
   } else
      debugger_log(&_dbgTemp[1],len); /* if the string ends with ";g;" */

   return len;
}

/* ------- io stuff */

#ifdef THINK_C

  void exit(int x)
  {
#pragma unused (x)
     ExitToShell();
  }

  /* make sure that this prototype is defined */
  extern int cs_exit(int);

 /* this is Symantec C's way of handling "startup" */
  extern int main(int, char**);

# if _USE_START_ && !_USE_APPLE_LIBRARIES_
   void __start(void)
   {
      char          *mainName = "main";

      main(1,&mainName);

      cs_exit(1);
   }
# endif

# if defined(env68k) && !TARGET_RT_MAC_68881

   double_t asin(double_t x)
   {
      double_t      temp = double_t_one-x;

      if (fabs(x)<=0.5)
         return atan(x/sqrt(double_t_one-x*x));
      temp *= temp;
      return atan(x/sqrt(2-2*x-temp));
   }

   double_t acos(double_t x) { return 2*atan(sqrt((double_t_one-x)/(double_t_one+x))); }

# endif

  long double cot(long double x) { return sin(x)/cos(x); }
#endif


/* -- rmalloc was removed for total lack of use (or need) -- */


/* non-debugging printf */

#if defined(__MRC__) || defined(__SC__) || defined(THINK_C)
  extern int _doprnt(const char *form, void *args, FILE *file);
#endif

UInt8 *psprintf(void *d, const char *form, ...)
{
   va_list      args;
   int          len;

   va_start(args,form);
#if VSNPRINTF_SUPPORTED
   len = vsnprintf((char*)(1+(UInt8*)d),255,form,args);
#else
   if ((len=vsprintf((char*)(1+(UInt8*)d),form,args))>255)
      len = 255;
#endif
   va_end(args);
   *(char*)d = len;

   return((UInt8*)d);
}

char *csprintf(char *d, const char *form, ...)
{
   va_list      args;

   va_start(args,form);
   vsprintf(d,form,args);
   va_end(args);

   return(d);
}

static const char   form_chars[] = "+-.0123456789Zbhlz%PXcdiopsuxEGefgn";

enum
{ /* modifier form chars */
   _fc_plus,
   _fc_minus,
   _fc_period,
   _fc_zero,
   _fc_one,
   _fc_two,
   _fc_three,
   _fc_four,
   _fc_five,
   _fc_six,
   _fc_seven,
   _fc_eight,
   _fc_nine,
   _fc_big_endian, /* special type */
   _fc_byte, /* special type */
   _fc_half, /* special type - 16 bits (short) */
   _fc_long,
   _fc_little_endian, /* special type */

  /* printing form chars */
   _fc_escape,
   _fc_Pascal_string,
   _fc_Hexadecimal,
   _fc_char,
   _fc_decimal,
   _fc_int,
   _fc_octal,
   _fc_pointer,
   _fc_string, /* C string */
   _fc_unsigned,
   _fc_hexadecimal,
   _fc_Exponential,
   _fc_Ranging_notation,
   _fc_exponential,
   _fc_floating,
   _fc_ranging_notation,
   _fc_notation
};

#define _CHAR_INDEX(s,c)                _char_index(s,c,sizeof(s))

static int _char_index(const char *s, int c, int z)
{
   char         *r;

   if ((r=memchr(s,c,z))==NULL)
      return NONE;
   return (long)r-(long)s;
}

enum
{ /* dsprintf flags */
   _pf_long     = 0x1
};

/* dsprintf has not been tested - don't use it */
UInt8 *dsprintf(void *_d, const char *t, const void *_s)
{
   /* to store the valid escape stuff before calling sprintf */
   char         fbuf[16];
   /* to store large data for endian conversion */
   char         dbuf[MAX(16,sizeof(long double))];

   const char   *e, *p;
   UInt8        *d = (UInt8*)_d, *s = (UInt8*)_d, c;
   int          len = strlen(t), size, index, endian, value;
   int          f, z;
   boolean      flags = 0;

   e = len+t;

   while(t<e && (t=memchr(p=t,'%',len))!=NULL)
   {
      memmove(d,p,z=(long)t-(long)p); /* copy the string in between */
      d += z;        /* increment the destination */
      z = 4;         /* default data size is four bytes */
      p = t;         /* this is so we can increment `t' without feeling guilty */
      endian = NONE; /* clear the endian specific */
      f = -1;        /* set up our valid escape (turns non-negative at first) */
      flags = 0;     /* and clear the flags */
      size = 0;      /* primarily for bounded strings */

      switch(index=_CHAR_INDEX(form_chars,c=*++t))
      {
      case _fc_zero:
      case _fc_one:
      case _fc_two:
      case _fc_three:
      case _fc_four:
      case _fc_five:
      case _fc_six:
      case _fc_seven:
      case _fc_eight:
      case _fc_nine:
        size *= 10; size += index-_fc_zero;
      case _fc_half:
      case _fc_long:
      case _fc_plus:
      case _fc_minus:
      case _fc_period:
      case _fc_big_endian:
      case _fc_little_endian:
         while((unsigned)(index=_CHAR_INDEX(form_chars,c=*t++))<_fc_escape)
         {
            switch(index)
            {
            case NONE:
               break;
            case _fc_one:
            case _fc_two:
            case _fc_three:
            case _fc_four:
            case _fc_five:
            case _fc_six:
            case _fc_seven:
            case _fc_eight:
            case _fc_nine:
               size *= 10; size += index-_fc_zero;
               break;
            case _fc_big_endian:
            case _fc_little_endian:
               endian = index;
               break;
            case _fc_half: z = 2; break;
            case _fc_byte: z = 1; break;
            case _fc_long:
               if (flags & _pf_long) z = 8; /* two `l's mean "long long" */
               flags |= _pf_long;
            default:
               if (f<=9)
                  fbuf[++f] = c;
               break;
            }
         }
         if (f>9)
            f = 9;
         if ((fbuf[++f]=c)==0)
            goto end; /* we could get termination at any time */

         fbuf[++f] = 0; /* and terminate our temporary format */

         if (index<=_fc_hexadecimal)
         {
            /* reflect whether the value is signed */
            index = index==_fc_hexadecimal ? NONE:index==_fc_decimal || index==_fc_int;

            if (z==1)
               value = index ? *(SInt8*)d:*(UInt8*)d;
            else if (z==2)
            {
               value = *(UInt16*)d;
               if (endian==_fc_big_endian)
                  value = index ? BES16_TO_NATIVE(value):BEU16_TO_NATIVE(value);
               else if (endian==_fc_little_endian)
                  value = index ? LES16_TO_NATIVE(value):LEU16_TO_NATIVE(value);
               else if (index)
                  value = (SInt16)value;
            } else if (z==4)
            {
               value = *(UInt32*)d;
               if (endian==_fc_big_endian)
                  value = BEU32_TO_NATIVE(value);
               else if (endian==_fc_little_endian)
                  value = LEU32_TO_NATIVE(value);
            } else if (z==8)
            {
               d += sprintf((char*)d,fbuf,value);
               s += 8;
               continue;
            }
            d += sprintf((char*)d,fbuf,value);
         } else
         {
            /* floating-point */
            if (flags & _pf_long)
            { /* 80, 96 or 128 bits (unreliable) */
               memmove(dbuf,s,z=sizeof(long double));
               if (endian!=NONE)
                  if ((TARGET_RT_LITTLE_ENDIAN==1)!=(endian==_fc_little_endian))
                     reverse_bytes(z,dbuf);
               d += sprintf((char*)d,fbuf,(long double*)dbuf);
            } else
            { /* 64-bit double (always - for compatibility) */
               memmove(dbuf,s,z=8);
               if (endian!=NONE)
                  if ((TARGET_RT_LITTLE_ENDIAN==1)!=(endian==_fc_little_endian))
                     reverse_bytes(z,dbuf);
               d += sprintf((char*)d,fbuf,(double*)dbuf);
            }
         }
         s += z;
         continue;

      case _fc_Pascal_string:
      case _fc_string:
         if (size==0)
         {
            d += (size=sprintf((char*)d,fbuf,s));
            ++size;
         } else
         {
 /*         d += snprintf((char*)d,size-(index==_fc_Pascal_string ? 1:0),fbuf,s); */
            d += snprintf((char*)d,size,fbuf,s);
         }
         s += size;
         continue;

      case _fc_decimal:
      case _fc_unsigned:
      case _fc_octal:
      case _fc_hexadecimal:
         /* nothing special; just let vsprintf handle it */
         break;

      case _fc_escape:
         break;
      default:
         if (c==0)
            goto end;
      }
      fbuf[0] = c, fbuf[1] = 0;
      d += vsprintf((char*)d,fbuf,(void*)&s);
      s += z;
   }
end:
   if (t==NULL)
      t = p;
   if (t<e && t!=NULL)
      memmove(d,t,(long)e-(long)t);

   return((UInt8*)_d);
}

#if defined(__MRC__) || defined(__SC__) || defined(THINK_C)
  int snprintf(char *buf, long max, const char *form, ...)
  {
     va_list      args;
     FILE         file;
     int          result;

     if ((unsigned)max>0x7FFFFFFF)
        max = 0x7FFFFFFF;
     file._cnt = max;
     file._base = file._ptr = (unsigned char*)buf;
     file._flag = 2, file._file = 0x28;

     va_start(args,form);
    /* terminate only if there is enough space in the buffer (their fault) */
     result = _doprnt(form,args,&file);
     if (file._ptr!=(unsigned char*)NULL && ((char*)file._ptr-buf)<max)
        *file._ptr = 0;
     va_end(args);

     return result;
  }

  int vsnprintf(char *buf, long max, const char *form, va_list args)
  {
     FILE         file;
     int          result;

     if ((unsigned)max>0x7FFFFFFF)
        max = 0x7FFFFFFF;
     file._cnt = max;
     file._base = file._ptr = (unsigned char*)buf;
     file._flag = 2, file._file = 0x28;

     result = _doprnt(form,args,&file);
     if (file._ptr!=(unsigned char*)NULL && ((char*)file._ptr-buf)<max)
        *file._ptr = 0;

     return result;
  }
#else

/* write for your own compiler software library */

#endif
