/*
 errors.c
  source file for MyCSeries error management and translation

  Version 0.4.1

  Created: January 9, 2006
  Revised: July 20, 2008
*/
/*
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 *  This source code is offered for use in the public domain.  You may use,
 * modify or distribute it freely.
 *
 *  This code is distributed in the hope that it will be useful but WITHOUT
 * ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY DISCLAIMED.
 *  This includes but is not limited to warranties of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.
 */

#include <cs_system.h>
#include <cs_files.h>
#include <cs_graphics.h>
#include <cs_text.h>

int translate_platform_error(int inErr, int inPlatform)
{
	if (inPlatform==_platform_MacOS || inPlatform==_platform_MacOSX)
	{
		static SInt16 err_table[] =
		{
			kCS_no_error,
			kCS_index_not_found, /* qErr */
			kCS_invalid_index, /* vTypErr */
			kCS_index_not_found, /* corErr */
			kCS_unimplemented_error, /* unimpErr - 4 */
			kCS_invalid_index, /* SlpTypeErr */
			kCS_unknown_error, /* ??? */
			kCS_unknown_error, /* ??? */
	
			kCS_debugger_not_installed, /* seNoDB - 8 */
			kCS_unknown_error, /* ??? */
			kCS_no_error, /* dsMacsBugInstalled (not an error) */
			kCS_no_error, /* dsDisassemblerInstalled (not an error) */
			kCS_no_error, /* dsHD20Installed (not an error) - 12 */
			kCS_no_error, /* dsExtensionsDisabled (not an error) */
			kCS_unknown_error, /* ??? */
			kCS_unknown_error, /* ??? */
	
			kCS_unknown_error, /* ??? - 16 */
			kCS_system_control_error, /* controlErr */
			kCS_system_status_error, /* statusErr */
			kCS_system_read_error, /* readErr */
			kCS_system_write_error, /* writErr - 20 */
			kCS_device_unit_reference, /* badUnitErr */
			kCS_device_unit_reference, /* unitEmptyErr */
			kCS_system_access_error, /* openErr */
			kCS_system_io_error, /* closErr - 24 */
			kCS_driver_removal_error, /* dRemovErr */
			kCS_driver_install_error, /* dInstErr */
			kCS_system_aborted, /* abortErr */
			kCS_driver_not_open, /* notOpenErr - 28 */
			kCS_device_unit_out_of_bounds, /* unitTblFullErr -- was kCS_device_unit_overflow */
			kCS_system_io_sw_error, /* dceExtErr */
			kCS_unknown_error, /* ??? */
			kCS_unknown_error, /* ??? - 32 */
			kFS_directory_full, /* dirFulErr */
			kFS_volume_full, /* dskFulErr */
			kFS_volume_not_found, /* nsvErr */
			kFS_hardware_io_error, /* ioErr - 36 */
			kFS_reserved_name, /* bdNamErr */
			kFS_bad_file_reference, /* fnOpnErr */
			kFS_file_eof_error, /* eofErr */
			kFS_negative_position, /* posErr - 40 */
			kFS_memory_or_slot_error, /* mFulErr */
			kFS_reference_overflow, /* tmfoErr */
			kFS_file_not_found, /* fnfErr */
			kFS_disk_write_protected, /* wPrErr - 44 */
			kFS_file_locked, /* fLckdErr */
			kFS_volume_locked, /* vLckdErr */
			kFS_deletion_of_open_file, /* fBsyErr */
			kFS_duplicate_name, /* dupFNErr - 48 */
			kFS_wfile_already_open, /* opWrErr */
			kCS_parameter_error, /* paramErr */
			kFS_file_reference_error, /* rfNumErr */
			kFS_get_position_error, /* gfpErr - 52 */
			kFS_volume_offline, /* volOffLinErr */
			kFS_file_permissions, /* permErr */
			kFS_volume_already_mounted, /* volOnLinErr */
			kFS_drive_not_found, /* nsDrvErr - 56 */
			kFS_unusable_signature, /* noMacDskErr */
			kFS_external_access, /* extFSErr */
			kFS_data_lost, /* fsRnErr */
			kFS_root_corrupt, /* badMDBErr - 60 */
			kFS_wfile_permission, /* wrPermErr */
			kCS_unknown_error, /* ??? */
			kCS_unknown_error, /* ??? */
			kCS_system_error, /* fontDecError, lastDskErr, noDriveErr - 64 */
			kCS_system_error, /* fontNotDeclared, offLinErr */
			kCS_system_error, /* fontSubErr, noNybErr */
			kCS_marker_not_found, /* noAdrMkErr */
			kCS_read_compare_verify, /* dataVerErr - 68 */
			kCS_checksum_error, /* badCksmErr */
			kCS_io_verify_error, /* badBtSlpErr */
			kCS_marker_not_found, /* noDtaMkErr */
			kCS_checksum_error, /* badDCksum - 72 */
			kCS_io_verify_error, /* badDBtSlp */
			kCS_insufficient_io_run, /* wrUnderrun */
			kCS_could_not_continue_run, /* cantStepErr */
			kCS_track_not_found, /* tk0BadErr - 76 */
			kCS_initialize_io_error, /* initIWMErr */
			kCS_unreachable_track, /* twoSideErr */
			kCS_system_control_error, /* spdAdjErr */
			kCS_io_verify_error, /* seekErr - 80 */
			kCS_sector_not_found, /* sectNFErr */
			kCS_sector_not_found, /* fmt1Err */
			kCS_io_sync_error, /* fmt2Err */
			kCS_io_verify_error, /* verErr - 84 */
	
			kCS_clock_read_error, /* clkRdErr */
			kCS_clock_write_error, /* clkWrErr */
			kCS_PRAM_error, /* prWrErr */
			kCS_PRAM_error, /* prInitErr - 88 */
	
			kCS_SCSI_receiver_error, /* rcvrErr */
			kCS_SCSI_received_break, /* breakRecd */
	
			kCS_socket_number_error, /* ddpSktErr */
			kCS_data_too_long, /* ddpLenErr - 92 */
			kCS_non_local_failure, /* noBridgeErr */
			KCS_protocol_attachment, /* lapProtErr */
			KCS_collisions_at_write, /* excessCollsns */
			kCS_serial_port_power, /* portNotPwr - 96 */
			kCS_driver_port_in_use, /* portInUse */
			kCS_driver_configuration, /* portNotCf */
			kCS_read_only_heap_zone_err, /* memROZError */
			kCS_clipboard_no_data, /* noScrapErr - 100 */
			kCS_unknown_error, /* ??? */
			kCS_clipboard_no_data, /* noTypeErr */
			kCS_unknown_error, /* ??? */
			kCS_unknown_error, /* ??? - 104 */
			kCS_unknown_error, /* ??? */
			kCS_unknown_error, /* ??? */
			kCS_unknown_error, /* ??? */
	
			kCS_memory_full_error, /* memFullErr - 108 */
			kCS_null_handle, /* nilHandleErr */
			kCS_bad_address, /* memAdrErr */
			kCS_free_block_error, /* memWZErr */
			kCS_purge_block_error, /* memPurErr - 112 */
			kCS_bad_address, /* memAZErr */
			kCS_block_verify_error, /* memPCErr */
			kCS_block_verify_error, /* memBCErr */
			kCS_block_verify_error, /* memSCErr - 116 */
			kCS_move_block_error, /* memLockedErr */
			kCS_unknown_error, /* ??? */
			kCS_unknown_error, /* ??? */
	
			kFS_directory_not_found, /* dirNFErr - 120 */
			kCS_system_error, /* tmwdoErr */
			kFS_move_into_offspring, /* badMovErr */
			kFS_op_not_supported, /* wrgVolTypErr */
			kFS_volume_offline, /* volGoneErr - 124 */
	
			kGR_mem_for_bitmap_update, /* updPixMemErr */
			kGR_missing_menubar_code, /* mBarNFnd */
			kCS_system_error, /* fsDSIntErr, hMenuFindErr */
			kCS_user_canceled, /* userCanceledErr - 128 */
			kCS_unknown_error, /* ??? */
			kCS_unknown_error, /* ??? */
			kCS_unknown_error, /* ??? */
			kCS_unknown_error, /* ??? - 132 */
			kCS_unknown_error, /* ??? */
			kCS_unknown_error, /* ??? */
			kCS_unknown_error, /* ??? */
			kCS_unknown_error, /* ??? - 136 */
			kCS_unknown_error, /* ??? */
			kCS_unknown_error, /* ??? */
			kCS_unknown_error, /* ??? */
			kCS_unknown_error, /* ??? - 140 */
			kCS_unknown_error, /* ??? */
			kCS_unknown_error, /* ??? */
			kCS_unknown_error, /* ??? */
	
			kCS_unknown_error, /* ??? - 144 */
			kGR_mem_for_render, /* noMemForPictPlaybackErr */
			kCS_unknown_error, /* ??? */
			kGR_region_overflow, /* rgnOverflowErr, rgnTooBigError */
			kGR_bm_unsupported_for_op, /* pixMapTooDeepErr - 148 */
			kCS_insufficient_stack, /* insufficientStackErr, nsStackErr */
			kGR_color_search_failed, /* cMatchErr */
			kGR_no_temporary_memory, /* cTempMemErr */
			kGR_no_structure_memory, /* cNoMemErr - 152 */
			kCS_invalid_index, /* cRangeErr */
			kGR_entry_protected, /* cProtectErr */
			kGR_invalid_device, /* cDevErr */
			kGR_bad_itab_resolution, /* cResErr - 156 */
			kGR_bad_pixel_format, /* cDepthErr */
		};
	
		if (inErr==-32615)
			return kCS_system_font_error; /* fontNotOutlineErr */
		if ((inErr=-inErr)<0)
			return kCS_system_error;
		if (inErr>157)
		{
			if (inErr<169)
				return kCS_unknown_error;
	
			switch(inErr)
			{
			case 6228: return kGV_driver_unsupported; /* kDMDriverNotDisplayMgrAwareErr */
			case 9108: return kGV_no_zoomed_video; /* kNotZVCapableErr */
	
			case 10930: return kGV_device_timing_error; /* kTimingChangeRestrictedErr */
			case 10935: return kGV_mirroring_unsupported; /* kVideoBufferSizeErr */
	
			case 10931: case 10932: case 10933: case 10934:
				return kGV_unknown_driver_error;
	
			default: return kCS_platform_software_error; /*kCS_unknown_error;*/
			}
		}
		return err_table[inErr];
	} else
		return kCS_unknown_error;
}
