/*
 glyph_drawing.h
  glyph mapper code ("included" by text.c);
   WARNING: not tested

  Created: January 10, 2006 11:44 PM EDST
  Revised: Sunday, June 24, 2007 10:20 AM EDST

January 24, 2006 4:52 PM
*/

/*
  PEL   = pixel typedef
  PELR  = pixel red component offset, if applicable
  PELG  = pixel green component offset, if applicable
  PELB  = pixel blue component offset, if applicable
  PELW  = pixel width (if less than 8bpp)
*/

#undef PELR
#undef PELG
#undef PELB
#undef PELW
#undef MAP_GLYPH

/* supports a couple pixel formats - 24 bit (3 byte) is no longer supported */
#ifndef DEPTH
# error "DEPTH" isn't defined.
#elif DEPTH==1
/* 1 bit (monochrome) */
# define PEL                UInt8
# define PELW               1
# define MAP_GLYPH          _map_glyph1

/* static defs all defined here */

#define MG_PARAMS           (cs_textspec *spec, cs_glyph_data *glyph, sfixed sx, sfixed sy)

typedef CALLBACK_API_C(int,map_glyph_proc) MG_PARAMS;

static int _map_glyph1 MG_PARAMS;
static int _map_glyph8 MG_PARAMS;
static int _map_glyph16 MG_PARAMS;
static int _map_glyph32 MG_PARAMS;

static map_glyph_proc       map_glyph_list[32] =
{
   _map_glyph1,
   NULL, /*_map_glyph2,*/
   NULL,
   NULL, /*_map_glyph4,*/
   NULL,
   NULL,
   NULL,
   _map_glyph8,
   NULL,
   NULL,
   NULL,
   NULL, /*_map_glyph12,*/
   NULL,
   NULL,
   _map_glyph16,
   _map_glyph16,
   NULL,
   NULL,
   NULL,
   NULL,
   NULL,
   NULL,
   NULL,
   NULL,
   NULL,
   NULL,
   NULL,
   NULL,
   NULL,
   NULL, /*_map_glyph30,*/
   NULL,
   _map_glyph32
};

#elif DEPTH==8
/* i8 (indexed) */
# define PEL                pixel8
# define MAP_GLYPH          _map_glyph8
#elif DEPTH==15 || DEPTH==16
/* 1:R5:G5:B5 || B5:G5:R5:1 ... */
# define PEL                pixel16
# define MAP_GLYPH          _map_glyph16
#elif DEPTH==32
 /* 8:R8:G8:B8 || B8:G8:R8:8 ... */
# define PEL                pixel32
# define PELR               PIXEL32_RED_OFFSET
# define PELG               PIXEL32_GREEN_OFFSET
# define PELB               PIXEL32_BLUE_OFFSET
# define MAP_GLYPH          _map_glyph32
#else
# error Depth not supported
#endif

/*
  methods in rendering 1 bit glyphs

 fast aliased (opaque)
   copy source (calculated fully or not) as index to table of pixel color
 aliased (transparent):
   copy pixel color for every one bit;
   otherwise do nothing
 fast anti-aliased (horizontal or vertical only):
   with 0x00, add 0x40 for every one bit on one line;
   0x00, no pixel: do nothing
   positive, half: shift constant, mask constant and or pixel color
   negative, full: copy pixel color
 anti-aliased (both horizontal and vertical):
   with x=0x80000004, subtract 0x10000001 from x for every one bit on two lines;
   x = 0x00, copy pixel color
   x>0: shift (byte)x, mask table index (short)x and or pixel color in table
   x<0: do nothing
*/
static int MAP_GLYPH MG_PARAMS
{
   UInt8        *tx = glyph->pixel_data;
   PEL          *r, *d, color = spec->privates[1];

   sfixed       xslop, yslop;
   ufixed       gw = glyph->bitmap_width<<16, gh = glyph->bitmap_height<<16;
   ufixed       gx = 0, gy = 0, lx = 0;
   sfixed       bx = RECT_WIDTH(&spec->bounds)<<16, by = RECT_HEIGHT(&spec->bounds)<<16;
   sfixed       x = sx, y;
   int          box_color = NONE, gp = (glyph->bitmap_width+7)>>3, b;

   if (spec->font.special&_ts_special_boxed)
   { /* no room reserved for a calculated box_color... damn. */
      cs_color_table    *ctab;
      int               depth = 0x3FFF&*(UInt16*)spec->privates;

      if ((ctab=*(cs_color_table**)(spec->privates+6))==(cs_color_table*)-1)
         ctab = (cs_color_table*)NULL;
      box_color = cs_rgb_color_to_pixel(cs_basic_colors[spec->boxColor],depth,_translate_color_closest,ctab);
   }

   if ((xslop=NFix30Mul(spec->xslop,glyph->scale))==0)
      xslop = 1;
   if ((yslop=NFix30Mul(spec->yslop,glyph->scale))==0)
      yslop = 1;

   if (xslop<0)
      lx += gw;
   if (yslop<0)
      gy += gh;

   /* calculate the pixel pointer, while clipping to bounds; compensate slops */
   {
      int       h, v;

      if ((h=sx-RECT_LEFT(&spec->bounds))<0)
         {lx += xslop*-h; h = 0;}
      if ((v=sy-RECT_TOP(&spec->bounds))<0)
         {gy += yslop*-v; v = 0;}
      b = 8-(h&7);
      r = h+(PEL*)(v*((UInt16*)spec->privates)[1]+*(UInt8**)(4+spec->privates));
   }

   if ((y=sy)>=by || (x=sx)>=bx) // || gx>=gw || gy>=gw
      return 0;

   do
   {
#ifdef PELW
      int       o = b;
      UInt8     p = *r>>o;
#endif
      d = r; gx = lx;
      do
      {
         if ((gx+=xslop)>=gw || gx<0 || (gy+=yslop)>=gh || gy<0)
            break;
         if (box_color==NONE)
         {
            if ((tx[(gy>>16)*gp+(gx>>3)]>>(gx&7))&1)
            {
#ifdef PELW
               if ((o-=PELW)&7)
                  p = (p<<PELW)+color;
               else
                  {*d++ = p; o = 8, p = 0;}
#else
               *d++ = color;
#endif
            } else
            {
#ifdef PELW
               if ((o-=PELW)&7)
                  p = (p<<PELW)+(*d>>PELW);
               else
                  {*d++ = p; o = 8, p = 0;}
#else
               ++d;
#endif
            }
         } else
         {
            if ((tx[(gy>>16)*gp+(gx>>3)]>>(gx&7))&1)
            {
#ifdef PELW
               if ((o-=PELW)&7)
                  p = (p<<PELW)+color;
               else
                  {*d++ = p; o = 8, p = 0;}
#else
               *d++ = color;
#endif
            } else
            {
#ifdef PELW
               if ((o-=PELW)&7)
                  p = (p<<PELW)+color;
               else
                  {*d++ = p; o = 8, p = 0;}
#else
               *d++ = box_color;
#endif
            }
         }
      } while(++x<bx);
      x = sx;
      r += ((UInt16*)spec->privates)[1];
   } while(++y<by);

   return 1;
}
