/*
 macintosh_cseries.c
  the cs_system code for MacOS Classic and Carbon.

  Version 0.4.2

  Created: March 12, 2004 6:05 PM EDST
  Revised: November 24, 2008 6:42 PM EDST

  Read the README and Revision History files for details.

A.P.:
	Dec 1 2005
		Moved the isqrt routines here (its function is universal).
	Dec 19 2005
		finf is now fnts.
		Got rid of the warnings (and a possible compatibility problem) by
			replacing if (h=) and if (!(h=)) with their _appropriate_
			equivalent.  This method doesnt even work with SC on actually
			increasing performance (its optimizer doesnt see it either way).
	Dec 24 2005
		Moved some of the faux ANSI routines here (they were kept in the
			substitute ansi_io.c source as a substitute for the Symantec C
			ANSI routines).
	Dec 26 2005
		Major rewriting for the TextSpec/FontSpec code.
	Jan 26 2006 14:55:24
    	Got version 0.1.10.2 to compile under Carbon
			(no PowerMac to test it, sorry).
	Jan 27 2006 7PM
		cs_screen_ are now cs_world_ and in here, as part of the
			MyCSeries library supplying those two variables.
		Added the version number globals so that the version number gets
			stored in the library file(s).
	Jul 10 2006 10PM
		Fixed the color list (there was one more than what was needed and
			the alert colors needed touching up).
	Jul 15 2006 1AM
		The enter key is now the entry key.
	Nov 19 2006 11AM
		Fixed a sign bug in animate_dialog_button.
	Jul 12 2007 12AM
		alert_user will now show the dialog if world port is not front window.
*/

#define __CSERIES_C
#define __MACINTOSHCSERIES_C

#include <Slots.h>
#include <Traps.h>
#include <Resources.h>
#include <Devices.h>
#include <Video.h>
#include <Dialogs.h>
#include <Gestalt.h>
#include <LowMem.h>
#include <MachineExceptions.h>

#include "macintosh_cseries.h"	/* {CIncludes}cseries.h */

UInt16				maccseries_version = 0xAA550000|MYCSERIES_VERSION;

/* BCD[major.8 minor.4 patch.4 development.16] -- .development is zero when "final" */
UInt32				mycseries_long_version = 0x00420001;

/* the traps and macros, and system codes were either obsoleted (removed), or put in
	macintosh_cseries.h */

/* globals */

RGBColor			cs_system_colors[] = {{52428,52428,-1}, {45056,45056,45056},
						{9830,9830,9830}, {21845,21845,21845}, {30583,30583,30583},
						{34733,34733,34733}, {43690,43690,43690}, {47840,47840,47840},
						{56797,56797,56797}, {61166,61166,61166}, {30583,13107,0},
						{52428,52428,52428}, {65535,61166,61166}, {52428,43690,43690}};

static cs_fontspec	defaultFontSpec = {kFontIDGeneva,0,0,9};

UInt32				cs_ms_per_second = 1000, cs_ms_second_sync = 0;

CGrafPtr			cs_world_port = (CGrafPtr)NULL;
GDHandle			cs_world_device = (GDHandle)NULL;

int					__cs_itmpa, __cs_itmpb, __cs_itmpc;

/* macros */

#ifdef env68k
  extern void SetA5ToCurrentA5(void) = {0x2A78, 0x904};
#else
# define SetA5ToCurrentA5()
#endif

/* code */

/* platform specific system routines dealing with strings */

int appendstrtomenu(MenuRef menu, short id)
{
	Handle			h;
	UInt16			*aptr, x;
	Byte			state;

	if ((h=GetResource('STR#',id))==(Handle)NULL)
		return -1;
	state = HGetState(h);
	HSetState(h,(state|kHandleLockedMask)&~kHandlePurgeableMask);
	aptr = (UInt16*)*h;
	if ((x=*aptr++)>0)
		do
		{
			AppendMenu(menu,*(Str255*)aptr);
			aptr = (UInt16*)((*(UInt8*)aptr)+1+(Ptr)aptr);
		} while(--x);
	HSetState(h,state);

	return 1;
}

/* fontspec loading/building routines */

int get_new_fontspec(cs_fontspec *spec, short fntsID, int x)
{
	Handle			h = (Handle)NULL;
	UInt16			*aptr = (UInt16*)NULL;
	int				result = 1;
	SInt16			fontID;
	Byte			state = 0;

	if (x<0 || (h=GetResource('fnts',fntsID))==(Handle)NULL)
		return -1;
	state = HGetState(h);
	HSetState(h,(state|kHandleLockedMask)&~kHandlePurgeableMask);
	MoveHHi(h);
	aptr = (UInt16*)*h;
	if (x>=*aptr++)
	{
	fail:
		*spec = defaultFontSpec;
		result = 0;
		goto end;
	} else if (x)
		do
		{
			aptr = (UInt16*)(5+(*(UInt8*)aptr)+(UInt8*)aptr);
		} while(--x);

	GetFNum((UInt8*)aptr,&fontID);
	aptr = (UInt16*)(1+(*(UInt8*)aptr)+(UInt8*)aptr);
	spec->id = fontID;
	*(UInt32*)&spec->special = *(UInt32*)aptr;

end:
	/* since the resource may get used a number of times, dont release it unless
	  memory is tight */
	if (h)
	{
		HSetState(h,state);
		/*ReleaseResource(h);*/
	}

	return result;
}

/* interface */
void get_window_frame(WindowRef window, Rect *screen_bounds)
{
	GrafPtr			old_port;

#if TARGET_API_MAC_CARBON
	GetPortBounds(GetWindowPort(window),screen_bounds);
#else
	*screen_bounds = GetWindowPort(window)->portRect;
#endif
	GetPort(&old_port);

	SetPortWindowPort(window);
	LocalToGlobal((Point*)&screen_bounds->top);
	LocalToGlobal((Point*)&screen_bounds->bottom);
	SetPort(old_port);
}

int wait_for_click_or_keypress(UInt32 maxMilliseconds, boolean waitUp)
{
	UInt64			timeOut, tms;
	EventRecord		event;
	boolean			inter = false;

	machine_milliseconds(timeOut);
	Add64U32(timeOut,maxMilliseconds);

	while(1)
	{
		GetNextEvent(mDownMask|keyDownMask|autoKeyMask,&event);
		if (event.what)
		{
			inter = true;
			break;
		}
		machine_milliseconds(tms);
		if (U64Gt(tms,timeOut))
			break;
	}
	if (!waitUp)
		goto end;
	/* wait for the user to release whatever theyre holding down */
	while(1)
	{
		GetNextEvent(mDownMask|mUpMask|keyDownMask|keyUpMask|autoKeyMask,&event);
		switch(event.what)
		{
		case nullEvent:
		case mouseUp:
		case keyUp:
			goto end;
		}
	}
end:
	FlushEvents(mDownMask|mUpMask|keyDownMask|keyUpMask|autoKeyMask,0);
	return inter;
}

int direct_set_colors(short first, short count, GDHandle gdh, ColorSpec *cs)
{
	VDSetEntryRecord	entryRec;
	ParamBlockRec		pb;

	if (gdh==(GDHandle)NULL)
		gdh = LMGetTheGDevice();
#if TARGET_API_MAC_CARBON
# pragma unused(first,count,gdh,cs,entryRec,pb)
	return -1; /* not yet implemented (sorry) */
#else
	entryRec.csStart = first;
	entryRec.csCount = count;
	entryRec.csTable = cs;
	memset(&pb,0,sizeof(pb.cntrlParam));

	pb.cntrlParam.ioCRefNum = (**gdh).gdRefNum;
	if ((*gdh)->gdType==directType)
		pb.cntrlParam.csCode = cscDirectSetEntries;
	else
		pb.cntrlParam.csCode = cscSetEntries;

	*(VDSetEntryRecord**)&pb.cntrlParam.csParam = &entryRec;

	return translate_platform_error(PBControlSync(&pb),_platform_MacOS);
#endif
}

/* Screen Saver */

typedef pascal short (*SAVCCall)(short);

SAVCCall				AfterDarkCaller = (SAVCCall)NULL;

void kill_screen_saver()
{
	short			err;
	long			result;

	if ((err=Gestalt('SAVR',&result))!=noErr)
		return;
	if (result&1)
	{
		if ((err=Gestalt('SAVC',&result))!=noErr)
			return;
		AfterDarkCaller = (SAVCCall)result;
		if (((SAVCCall)result)(3))
			return;
/*atexit(revive_screen_saver);*/
	}
}

void revive_screen_saver()
{
	if (AfterDarkCaller!=(SAVCCall)NULL)
		AfterDarkCaller(2);
}

/* Dialogs
	- most of them were moved to myDialogs.c
	- these are the ones that cant be moved because they use low level routines
		that come after them in this file

  the out of memory string is pre-supplied so no handles are used in the process
   (good in the case the heap is full or corrupt) - this should virtually eliminate
   #25 Errors it was static until I needed the string to be flexible over different
   projects using this file; it now lies in shell.c
*/
extern const UInt8	_out_of_memory[];

void alert_user(int type, short strID, int index, short err)
{
	Str255			astr;
	Str31			bstr;

	CGrafPtr		old_port = GetWindowPort(FrontWindow());
	const UInt8		*prompt;

	psprintf(bstr,"%d", err);
	if (strID==128 && index==0)
		prompt = _out_of_memory;
	else
		prompt = getpstr(astr,strID,index);

	/* make sure the user can see the frickin cursor! */
	show_cursor();
	stop_interface_fade();

	/* if theres no screen window, put up a dialog, otherwise use the screen window */
	if (cs_world_port==NULL || (old_port!=NULL && cs_world_port!=old_port
			&& GetNextWindow(GetWindowFromPort(old_port))!=NULL))
	{
		ParamText(prompt,bstr,empty,empty);
		Alert(type+128,NULL);
	} else
	{
		KeyMap		keyMap;
		Rect		arect, brect;
		RGBColor	old_back, old_fore;

		GDHandle	old_device;
		int			button_press = -1, key_press = -1;
		short		x, y;

#if TARGET_API_MAC_CARBON
		GetPortBounds(cs_world_port,&arect);
#else
		arect = cs_world_port->portRect;
#endif
		GetGWorld(&old_port,&old_device);
		SetGWorld(cs_world_port,cs_world_device);
		GetForeColor(&old_fore);
		GetBackColor(&old_back);

		TextMode(srcOr);
		TextFace(0);
		TextSize(0);
		TextFont(0);
		PenSize(1,1);

		arect.bottom = 120+(arect.top=(arect.bottom>>1)-100);
		arect.right = 416+(arect.left=(arect.right>>1)-200);
		brect = arect;
		RGBForeColor(cs_system_colors+_system_alert_frame_color);
		FrameRect(&brect); ++brect.top, ++brect.left;
		RGBBackColor(cs_system_colors+_system_alert_content_color);
		EraseRect(&brect);

		RGBForeColor(cs_system_colors+_system_alert_tl_color);
		MoveTo(brect.right-2,y=--brect.top);
		LineTo(x=--brect.left,y);
		LineTo(x,y=brect.bottom-2);
		RGBForeColor(cs_system_colors+_system_alert_br_color);
		MoveTo(x,++y);
		LineTo(x=brect.right-1,y);
		LineTo(x,brect.top);

		RGBForeColor(&cs_rgb_black);

		arect.bottom = 68+(arect.top+=8), arect.left += 12, arect.right -= 12;
		TETextBox(prompt+1,prompt[0],&arect,teJustCenter);
		MoveTo(arect.left,arect.top+102);
		DrawString("\pID = ");
		DrawString(bstr);
		MoveTo(arect.left+114,arect.top+102);
		DrawString("\pClick or press return/enter");
		if (type!=fatalError)
			DrawString("\p to continue.");
		else
		{
			*(SInt32*)&bstr = '\3...';
			DrawString(bstr);
		}

		while(button_press<=0 && key_press<=0)
		{
			if ((button_press==0 && Button()) || (button_press<0 && !Button()))
				++button_press;
			GetKeys(keyMap);
			if ((key_press==0 && ((keyMap[1]&(1L<<28))|(keyMap[2]&(1L<<20))))
					|| (key_press<0 && !(((keyMap[1]&(1L<<28))|(keyMap[2]&(1L<<20))))))
				++key_press;
		}
		/* wait for depress */
		do
		{
			GetKeys(keyMap);
		} while(Button() || (((keyMap[1]&(1L<<28))|(keyMap[2]&(1L<<20)))));

		PaintRect(&brect);
#if TARGET_API_MAC_CARBON
		InvalWindowRect(GetWindowFromPort(cs_world_port),&brect);
#else
		InvalRect(&brect);
#endif
		FlushEvents(mDownMask|mUpMask|keyDownMask|keyUpMask|autoKeyMask,0);

		RGBForeColor(&old_fore);
		RGBBackColor(&old_back);
		SetGWorld(old_port,old_device);
	}
	if (type==fatalError)
		exit(-1);
}

/* dialogs */

int extract_int_from_text_item(DialogRef d, int i)
{
	UInt8			astr[257];
	SInt16			it;
	Handle			ih;
	Rect			ir;

	GetDialogItem(d,i,&it,&ih,&ir);
	GetDialogItemText(ih,astr);
	astr[1+astr[0]] = 0;

	return atoi((char*)&astr[1]);
}

void insert_int_into_text_item(DialogRef d, int i, int v)
{
	Str255			astr;
	SInt16			it;
	Handle			ih;
	Rect			ir;

	GetDialogItem(d,i,&it,&ih,&ir);
	psprintf(astr,"%d", v);
	SetDialogItemText(ih,astr);
}

void animate_dialog_button(DialogRef d, int i)
{
	short			it;
	Handle			ih;
	Rect			ir;

	UInt32			timeOut;

	GetDialogItem(d,i,&it,&ih,&ir);

	timeOut = TickCount()+6;
	HiliteControl((ControlRef)ih,1);
	do
	{
		global_idle_proc();
	} while(!((timeOut-TickCount())&SINT32_MIN));

	HiliteControl((ControlRef)ih,0);

	timeOut = TickCount()+2;
	do
	{
		global_idle_proc();
	} while(!((timeOut-TickCount())&SINT32_MIN));
}

/*	device/driver routines */

#if !TARGET_API_MAC_CARBON
OSErr get_driver_name(UInt8 slot, Str63 str)
{
	SpBlock			spb;
	long			length;
	OSErr			err = 0;

	str[0] = 0;
	*(SInt32*)&spb.spCType = 0, *(SInt32*)&spb.spDrvrHW = 0,
		*(SInt32*)&spb.spID = 0, spb.spSlot = slot, spb.spCategory = 1; /* ??? */

	if ((err=SNextTypeSRsrc(&spb))!=noErr)
		goto err;
	spb.spID = 2;
	if ((err=SGetCString(&spb))!=noErr)
		goto err;
	if ((length=strlen((char*)spb.spResult))>255)
		length = 255;
	str[0] = length;
	BlockMoveData((Ptr)spb.spResult,&str[1],length);
	DisposePtr((Ptr)spb.spResult);
err:
	return err;
}

unsigned char *GetDCtlName(short refNum)
{
	DCtlHandle		dctlh;
	DRVRHeader		*drvr;

	if ((dctlh=GetDCtlEntry(refNum))==(DCtlHandle)NULL
			|| (drvr=(DRVRHeader*)(*dctlh)->dCtlDriver)==(DRVRHeader*)NULL)
		return (unsigned char*)NULL;
	return drvr->drvrName;
}

short GetSlotFromGDevice(GDHandle gd)
{
	AuxDCEHandle	dctl;

	if ((dctl=(AuxDCEHandle)GetDCtlEntry((*gd)->gdRefNum))==(AuxDCEHandle)NULL)
		return 0;
	return (*dctl)->dCtlSlot;
}
#endif /* !TARGET_API_MAC_CARBON */

/* system routines */

static int					cs_init_flags = 0, cs_exit_count = 0, cs_exit_code;

/* memory/heaps */

Ptr new_platform_ptr(long z)
{
	if ((unsigned)z>0x7FFFFFF0)
		return (Ptr)NULL;

	return NewPtr(z);
}

int dispose_platform_ptr(Ptr p)
{
	if (p==(Ptr)NULL)
		return kCS_null_pointer;
	if (p==(Ptr)-1)
		return kCS_bad_address;
	DisposePtr(p);

	return translate_platform_error(MemError(),_platform_MacOS);
}

long get_platform_ptr_size(Ptr p)
{
	if (p==(Ptr)NULL)
		return kCS_null_pointer;
	if (p==(Ptr)-1)
		return kCS_bad_address;
#ifdef env68k
	return InlineGetPtrSize(p);
#else
	{
		long		size;
		int			error;

		size = GetPtrSize(p);
		if ((error=translate_platform_error(MemError(),_platform_MacOS))<0)
			return error;
		return size;
	}
#endif
}

int set_platform_ptr_size(Ptr p, long z)
{
	if (p==(Ptr)NULL)
		return kCS_null_pointer;
	if (p==(Ptr)-1)
		return kCS_bad_address;
	SetPtrSize(p,z);

	return translate_platform_error(MemError(),_platform_MacOS);
}

Handle new_platform_handle(long z, boolean lock)
{
	Handle			h;

	if ((unsigned)z>0x7FFFFFF0)
		return (Handle)NULL;
	if ((h=NewHandle(z))!=(Handle)NULL && lock)
		HLock(h);

	return h;
}

int dispose_platform_handle(Handle h)
{
	if (h==(Handle)NULL)
		return kCS_null_handle;
	if (h==(Handle)-1)
		return kCS_bad_address;
	DisposeHandle(h);

	return translate_platform_error(MemError(),_platform_MacOS);
}

long get_platform_handle_size(Handle h)
{
	if (h==(Handle)NULL)
		return kCS_null_handle;
	if (h==(Handle)-1)
		return kCS_bad_address;
#ifdef env68k
	return InlineGetHandleSize(h);
#else
	{
		long		size;
		int			error;

		size = GetHandleSize(h);
		if ((error=translate_platform_error(MemError(),_platform_MacOS))<0)
			return error;
		return size;
	}
#endif
}

int set_platform_handle_size(Handle h, long z)
{
	if (h==(Handle)NULL)
		return kCS_null_handle;
	if (h==(Handle)-1)
		return kCS_bad_address;
	SetHandleSize(h,z);

	return translate_platform_error(MemError(),_platform_MacOS);
}

int append_platform_handle(platform_handle h, long added)
{
	long		size;
	int			error;

	if (h==(Handle)NULL)
		return kCS_null_handle;
	if (h==(Handle)-1)
		return kCS_bad_address;

	size = GetHandleSize(h);
	if ((error=translate_platform_error(MemError(),_platform_MacOS))<0)
		return error;
	SetHandleSize(h,size+added);

	return translate_platform_error(MemError(),_platform_MacOS);
}

int get_platform_handle_state(Handle h, boolean *lock, boolean *purge)
{
	int			error;
	Byte		state;

	if (h==(Handle)NULL)
		return kCS_null_handle;
	if (h==(Handle)-1)
		return kCS_bad_address;

	state = HGetState(h);
	if ((error=translate_platform_error(MemError(),_platform_MacOS))<0)
		return error;
	if (lock)
		*lock = state&kHandleLockedMask;
	if (purge)
		*purge = state&kHandlePurgeableMask;

	return translate_platform_error(MemError(),_platform_MacOS);
}

int set_platform_handle_state(Handle h, boolean lock, boolean purge)
{
	if (h==(Handle)NULL)
		return kCS_null_handle;
	if (h==(Handle)-1)
		return kCS_bad_address;
	HSetState(h,(lock ? kHandleLockedMask:0)|(purge ? kHandlePurgeableMask:0));

	return translate_platform_error(MemError(),_platform_MacOS);
}

int lock_platform_handle(Handle h)
{
	if (h==(Handle)NULL)
		return kCS_null_handle;
	if (h==(Handle)-1)
		return kCS_bad_address;
	HLock(h);

	return translate_platform_error(MemError(),_platform_MacOS);
}

int unlock_platform_handle(Handle h)
{
	if (h==(Handle)NULL)
		return kCS_null_handle;
	if (h==(Handle)-1)
		return kCS_bad_address;
	HUnlock(h);

	return translate_platform_error(MemError(),_platform_MacOS);
}

int move_platform_handle_high(Handle h)
{
	if (h==(Handle)NULL)
		return kCS_null_handle;
	if (h==(Handle)-1)
		return kCS_bad_address;
	MoveHHi(h);

	return translate_platform_error(MemError(),_platform_MacOS);
}

/* private code */

static OSStatus __RTExit(ExceptionInformation *theException);

#define MAXIMUM_EXIT_ROUTINES		100

static VoidRoutineProc		_exitRoutines[MAXIMUM_EXIT_ROUTINES*2];
static UInt32				_disableCodeFilter = 0;
static short				_exitRoutineCount = 0;

#if !TARGET_API_MAC_CARBON
static const UInt8			_DSErrCodeID[128] =
	{1, 3, 2, 7,10,10,12,10,10,10, 1, 3,18,66,13, 9,
	17,17,17,17,17,17,17,17, 0,15,20, 5,-1,-1,-1,-1,
	 6,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
	-1,-1,19,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
	-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
	 9,-1,11,16,14,14,14,14,14, 8,-1,-1,-1,-1,-1,-1,
	-1,-1,13,-1, 4,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
	-1,-1,-1,-1,-1,-1,21,-1,-1,-1,-1,-1,-1,-1,-1,22};
#endif

static VoidRoutineProc		__ES_JUMP = (VoidRoutineProc)NULL; /* always NULL under Carbon */

int atexittoshell(VoidRoutineProc proc, int flags)
{
	VoidRoutineProc		*d;

	if (cs_init_flags&kCSI_inhibit_atexittoshell)
		return kCS_flag_suppressed;
	if (_exitRoutineCount>=MAXIMUM_EXIT_ROUTINES)
	{
		DebugStr("\pToo many atexittoshell routines.");
		return -1;
	}
	d = _exitRoutines+(2*_exitRoutineCount++);
	*d++ = proc; *d++ = (VoidRoutineProc)flags;
	return 0;
}

int cs_runtime(UInt32 flags)
{
#pragma unused(flags) /* not used yet */
	if (cs_init_flags&kCSI_uses_SDL_timer)
	{
		/* do we do anything here?  machine_milliseconds doesnt use SDL in this case */
	}
	return 0;
}

int cs_initialize(const cs_init_params *params)
{
	ExceptionHandlerTPP		exitHandler;

	unsigned long	currentDate;
	SInt32			size, count, temp;

	if (cs_init_flags!=0)
		return cs_init_flags;
	assert(__ES_JUMP==(VoidRoutineProc)NULL);
	if (params==(const cs_init_params*)NULL || (size=params->init_params_size)<=0)
		goto end;
	if ((size-=sizeof(params->flags))<=0)
		goto end;

	cs_init_flags = params->flags;
	if (params->flags&kCSI_approximate_ms)
	{
		unsigned long		firstDate;
		UInt64				firstmcs, lastmcs;

	/* using the DateTime seconds, sync by waiting until this second is over */
	/* this obviously wont help if the second clock timer is less accurate */
		GetDateTime(&firstDate);
		do{GetDateTime(&currentDate);} while(currentDate==firstDate);
		GetDateTime(&firstDate);
		do{GetDateTime(&currentDate);} while(currentDate==firstDate);
	/* time this second */
		Microseconds((UnsignedWide*)&firstmcs);
		GetDateTime(&firstDate);
		do{GetDateTime(&currentDate);} while(currentDate==firstDate);
		Microseconds((UnsignedWide*)&lastmcs);
		USub64(lastmcs,firstmcs);
		cs_ms_per_second = ((Int64Lo(lastmcs)/500)+1)>>1;
		cs_ms_second_sync = Int64Lo(firstmcs)/1000;
	}
	if ((size-=sizeof(params->init_master_count))<=0)
		goto end;
	if ((count=params->init_master_count)<=63)
		count = 64;
#if TARGET_API_MAC_CARBON
	MoreMasterPointers(count);
#else
	if ((count=params->init_master_count-64)<0)
		count = 0;
	if (count>32767)
		temp = 32767; /*return kCS_miserable_failure;*/
	else
		temp = count;

	{
		THz			zone;
		SInt16		oldValue;

		zone = GetZone();
		oldValue = zone->moreMast;
		zone->moreMast = temp;

		while((count-=temp)>0)
			MoreMasters();

		if (count+=temp)
			{zone->moreMast = count; MoreMasters();}

		zone->moreMast = oldValue;
	}
#endif
end:
#if env68k
# pragma unused (exitHandler)
	__ES_JUMP = (VoidRoutineProc)GetToolTrapAddress(_ExitToShell);
	SetToolTrapAddress((UniversalProcPtr)__RTExit,_ExitToShell);
#else
	exitHandler = NewExceptionHandlerUPP(__RTExit);
# if TARGET_API_MAC_CARBON
#  pragma unused (temp)
	InstallExceptionHandler(exitHandler);
# else
	__ES_JUMP = (VoidRoutineProc)GetToolTrapAddress(_ExitToShell);
	SetToolTrapAddress((UniversalProcPtr)exitHandler,_ExitToShell);
# endif
#endif
	cs_init_flags |= (long)0x80000000;  /* make sure it is not zero */

	return 0;
}

int cs_exit(int x);
int cs_exit(int x)
{
	++cs_exit_count, cs_exit_code = x;
	ExitToShell();
	exit(x);

	return 0; /*kCS_miserable_failure;*/
}

static OSStatus __RTExit(ExceptionInformation *theException)
{
	VoidRoutineProc		*aptr = _exitRoutines+_exitRoutineCount*2;
	VoidRoutineProc		routine;

	long			disableCode = 0, code = -1;

	SetA5ToCurrentA5();	/* macroed out on non-68k */

#if TARGET_API_MAC_CARBON
	if (theException!=(ExceptionInformation*)NULL)
		code = theException->info.memoryInfo->theError;
	switch(code)
	{
	case kIllegalInstructionException: disableCode = kDFIllegal; break;
	case kTrapException: disableCode = kDFTrap|kDFPatchTrap; break;

	case kAccessException:
	case kUnmappedMemoryException:
	case kExcludedMemoryException:
	case kReadOnlyMemoryException:
		disableCode = kDFAccessFault; break;
	case kUnresolvablePageFaultException:
		disableCode = kDFIOFailure; break;

	case kPrivilegeViolationException: disableCode = kDFPriviledgeViolation; break;
	case kTraceException: disableCode = kDFTrap; break;
	case kInstructionBreakpointException: disableCode = kDFTrap; break;
	case kDataBreakpointException: disableCode = kDFTrap; break;
	case kIntegerException: disableCode = kDFBadIntegerMath; break;
	case kFloatingPointException: disableCode = kDFFPException; break;

	case kStackOverflowException:   /* Optional, may be implemented as kAccessException on some systems.*/
		disableCode = kDFStackEnteredHeap; break;
	case kTaskTerminationException:   /* Obsolete*/
	case kTaskCreationException:   /* Obsolete*/
		break;

	/* May occur when a task is in little endian mode or created with kMPTaskTakesAllExceptions */
	case kDataAlignmentException:
		disableCode = kDFBadAddress; break;
	}
#else
/* theException is not used in this case */
# pragma unused(theException)
	if (cs_init_flags)
		if ((disableCode=LMGetDSErrCode())>0)
		{
			if ((UInt16)disableCode>127)
				goto nada;
			code = _DSErrCodeID[disableCode-1];
			if ((disableCode=0x800000|(1<<(code&63))|(1<<(code>>6)))<0)
				goto end;	/* this is a FATAL error, D0 N0T run any exit routines */
		} else
			nada: disableCode = 0;
#endif

	/* make sure EVERYTHING is caught */
	_disableCodeFilter = disableCode |= _disableCodeFilter;
	if (_exitRoutineCount>0)
		do
		{
			code = (long)(*--aptr);
			if ((routine=*--aptr)!=(VoidRoutineProc)NULL)
			{
				*aptr = nil;			/* to make sure it doesn... */
				if ((code&=disableCode)==0)
					(routine)();
			}
		} while(--_exitRoutineCount);
end:
#if !TARGET_API_MAC_CARBON
	if (__ES_JUMP==(VoidRoutineProc)NULL || (long)__ES_JUMP==-1)
		DebugStr("\pES_JUMP is invalid!");
	SetToolTrapAddress((UniversalProcPtr)__ES_JUMP,_ExitToShell);

	ExitToShell();
#endif
	return 0;
}
