/*
 strings.c

  Version 0.4.1

  Created: December 26, 2005
  Revised: July 20, 2008

  Read the README file for details.

Monday, December 26, 2005
  moved the universal string routines here
Saturday, November 25, 2006
  getcstr should now be consistent with countstrlist (the count is read unsigned).
Saturday, June 14, 2008 8:30 AM
  "improved" cs_MacRoman_uppercase.  O-slash (0xBF->0xBE), delta (0xB6->0xC6).
*/

#include <cs_strings.h>
#include <ctype.h>

/* globals */

const UInt8			empty[16] = {0, 0};

/* strings */

UInt8 *pstrcpy(void *d, const void *s)
{
	if (d!=NULL)
		memmove(d,s,1+*(UInt8*)s);

	return d;
}

UInt8 *pstrcat(register void *d, register const void *s)
{
	int				dlen = *(UInt8*)d, slen = *(UInt8*)s;
	int				len = dlen;

	if ((len+=slen)>255)
		*(UInt8*)d = len = 255;
	memmove(1+dlen+(UInt8*)d,1+(UInt8*)s,(*(UInt8*)d=len)-slen);
	return d;
}

#ifndef env68k
/* copies source to destination with `max' clipping the outgoing length */
UInt8 *cpstrcpy(void *dst, const void *src, short max)
{
	const UInt8		*s;
	UInt8			*d;
	int				x, len;

	if ((d=(UInt8*)dst)==NULL || d==(UInt8*)(-1))
		return NULL;
	*d++ = 0;
	if ((s=src)==NULL || s==(UInt8*)(-1))
		return NULL;
	if ((len=max)==0)
		goto end;
	if (max>255)
		len = 255;
	x = len;

	do
	{
		if (0==(*d++=*s++))
			break;
	} while(--x!=0);

	*(UInt8*)dst = len-=x;
end:
	return dst;
}

/* memory */

static const UInt8	cs_MacRoman_uppercase_table[] =
{
	0x00,0x01,0x02,0x03,0x04,0x05,0x06,0x07,0x08,0x09,0x0A,0x0B,0x0C,0x0D,0x0E,0x0F,
	0x10,0x11,0x12,0x13,0x14,0x15,0x16,0x17,0x18,0x19,0x1A,0x1B,0x1C,0x1D,0x1E,0x1F,
	0x20,0x21,0x22,0x23,0x24,0x25,0x26,0x27,0x28,0x29,0x2A,0x2B,0x2C,0x2D,0x2E,0x2F,
	0x30,0x31,0x32,0x33,0x34,0x35,0x36,0x37,0x38,0x39,0x3A,0x3B,0x3C,0x3D,0x3E,0x3F,
	0x40,0x41,0x42,0x43,0x44,0x45,0x46,0x47,0x48,0x49,0x4A,0x4B,0x4C,0x4D,0x4E,0x4F,
	0x50,0x51,0x52,0x53,0x54,0x55,0x56,0x57,0x58,0x59,0x5A,0x5B,0x5C,0x5D,0x5E,0x5F,
	0x60,0x41,0x42,0x43,0x44,0x45,0x46,0x47,0x48,0x49,0x4A,0x4B,0x4C,0x4D,0x4E,0x4F,
	0x50,0x51,0x52,0x53,0x54,0x55,0x56,0x57,0x58,0x59,0x5A,0x7B,0x7C,0x7D,0x7E,0x7F,
	0x80,0x81,0x82,0x83,0x84,0x85,0x86,0xE7,0xCB,0xE5,0x80,0xCC,0x81,0x82,0x83,0xE9,
	0xE6,0xE8,0xEA,0xED,0xEB,0xEC,0x84,0xEE,0xF1,0xEF,0x85,0xCD,0xF2,0xF4,0xF3,0x86,
	0xA0,0xA1,0xA2,0xA3,0xA4,0xA5,0xA6,0xA7,0xA8,0xA9,0xAA,0xAB,0xAC,0xAD,0xAE,0xAF,
	0xB0,0xB1,0xB2,0xB3,0xB4,0xB5,0xC6,0xB7,0xB8,0xB8,0xBA,0xBB,0xBC,0xBD,0xAE,0xAF,
	0xC0,0xC1,0xC2,0xC3,0xC4,0xC5,0xC6,0xC7,0xC8,0xC9,0xCA,0xCB,0xCC,0xCD,0xCE,0xCE,
	0xD0,0xD1,0xD2,0xD3,0xD4,0xD5,0xD6,0xD7,0xD9,0xD9,0xDA,0xDB,0xDC,0xDD,0xDE,0xDF,
	0xE0,0xE1,0xE2,0xE3,0xE4,0xE5,0xE6,0xE7,0xE8,0xE9,0xEA,0xEB,0xEC,0xED,0xEE,0xEF,
	0xF0,0xF1,0xF2,0xF3,0xF4,0xF5,0xF6,0xF7,0xF8,0xF9,0xFA,0xFB,0xFC,0xFD,0xFE,0xFF
};

const char			*cs_MacRoman_uppercase = (char*)cs_MacRoman_uppercase_table;

void *memchrbit(const void *_s, int _c, size_t _n)
{
	long			n;

	if ((n=_n)>0)
	{
		register const char	*s = (const char*)_s;
		register char		c = _c;

		do
		{
			if ((char)(*s++&c))
				return (void*)(s-1);
		} while(--n);
	}
	return NULL;
}

long memcnt(const void *_s, int _v, size_t _n)
{
	long			n, c = 0;

	if ((n=_n)>0)
	{
		register const char	*s = (const char*)_s;
		register char		v = _v;

		do
		{
			if (*s++==v)
				++c;
		} while(--n);
	}
	return c;
}

long memcntbit(const void *_s, int _v, size_t _n)
{
	long			n, c = 0;

	if ((n=_n)>0)
	{
		register const char	*s = (const char*)_s;
		register char		v = _v;

		do
		{
			if ((char)(*s++&v))
				++c;
		} while(--n);
	}
	return c;
}

void memrem(UInt8 *p, long c, UInt8 *t)
{
	if (c<=0)
		return;
	p += c;
	if (c&1)
		*p = t[*--p];
	if ((c>>=1)==0)
		return;
	do
	{
		*p = t[*--p]; *p = t[*--p];
	} while(--c);
}

int tblcmp(const char *_sa, const char *_sb, const char *_t)
{
	register const UInt8	*sa = (const UInt8*)_sa;
	register const UInt8	*sb;
	register const UInt8	*t = (const UInt8*)_t;
	register UInt8			a, b;

	if ((sb=(const UInt8*)_sb)==(const UInt8*)NULL)
		return (sa!=(const UInt8*)NULL) ? *sa:0;
	if (sa==(const UInt8*)NULL)
		return -*sb;
	do
	{
		b = *sb++;
		if ((a=*sa++)==0)
			break;
	} while(t[b]==a);

	return a-t[b];
}

/*
static void _tblcpy(UInt8*, const UInt8*, const UInt8*, long, UInt8, UInt8);
static void _tblcpy(UInt16*, const UInt16*, const UInt16*, long, UInt16, UInt16);
static void _tblcpy(UInt32*, const UInt32*, const UInt32*, long, UInt32, UInt32);
*/

#ifdef cplusplus
template <class T>
inline void __tblcpy(T *d, const T *s, const T *t, long n, T z, T v)
{
	if (n<=0)
		return;
	if (z<(1L<<(sizeof(T)<<3))-1)
	{
		do
		{
			register T		x = *s++;

			if (x>z)
				x = v;
			*d++ = t[x];
		} while(--n);
		return;
	}
	if (n & 1)
		*d++ = t[*s++];
	if ((n>>=1)==0)
		return;
	do
	{
		*d++ = t[*s++]; *d++ = t[*s++];
	} while(--n);
}
# define _tblcpy(d,s,t,n,z,v,T) __tblcpy(d,s,t,n,z,v)
#else
# define _tblcpy(d,s,t,n,z,v,T) do{ if(n<=0) return; \
	if(z<(1L<<(sizeof(T)<<3))-1) \
		do{ \
			register T	x = *s++; \
			if(x>z) x = v; \
			*d++ = t[x]; \
		} while(--n); \
	else \
	{	if(n&1) *d++ = t[*s++]; \
		if((n>>=1)==0) return; \
		do{*d++ = t[*s++]; *d++ = t[*s++];} while(--n); \
	} } while(0)
#endif

void tblcpy(UInt8 *d, const UInt8 *s, long n, const UInt8 *t)
{
	_tblcpy(d,s,t,n,(UInt8)255,(UInt8)0,UInt8);
}

void tblcpy2(UInt16 *d, const UInt16 *s, long n, const UInt16 *t, UInt16 z, UInt16 v)
{
	_tblcpy(d,s,t,n,z,v,UInt16);
}

void tblcpy4(UInt32 *d, const UInt32 *s, long n, const UInt32 *t, UInt32 z, UInt32 v)
{
	_tblcpy(d,s,t,n,z,v,UInt32);
}

char *tblchr(const char *_s, int _e, size_t _n, const char *t)
{
	long			n;

	if ((n=_n)>0)
	{
		register const UInt8	*s = (const UInt8*)_s;
		register char			e = _e;

		do
		{
			if (e==t[*s++])
				return (void*)(s-1);
		} while(--n);
	}
	return (char*)NULL;
}

char *tblchrbit(const char *_s, int _m, size_t _n, const char *t)
{
	long			n;

	if ((n=_n)>0)
	{
		register const UInt8	*s = (const UInt8*)_s;
		register char			m = _m;

		do
		{
			if ((char)(m&t[*s++]))
				return (void*)(s-1);
		} while(--n);
	}
	return (char*)NULL;
}

long tblcnt(const char *_s, int _e, size_t _n, const char *_t)
{
	long			n, c = 0;

	if ((n=_n)>0)
	{
		const UInt8		*s = (const UInt8*)_s, *t = (const UInt8*)_t;
		UInt8			e = _e;

		do
		{
			if (e==t[*s++])
				++c;
		} while(--n);
	}
	return c;
}

long tblcntbit(const char *_s, int _m, size_t _n, const char *t)
{
	long			n, c = 0;

	if ((n=_n)>0)
	{
		register const UInt8	*s = (const UInt8*)_s;
		register char			m = _m;

		do
		{
			if ((char)(m&t[*s++]))
				++c;
		} while(--n);
	}
	return c;
}
#endif

/* string lists */

#if TARGET_OS_MAC
# include <LowMem.h>
# include <Resources.h> // #include's MacMemory.h

# define _scan_string(s,z) do{Handle _h; \
	if(x<0 || (_h=GetResource('STR#',str_id))==(Handle)NULL) {z=0; break;} \
	s=(UInt8*)*_h; if(*((UInt16*)s)++<(unsigned)++x) {z=0; break;} \
	z=*s++; if(--x==0) break; do{s+=z; z=*s++;}while(--x);}while(0)

UInt8 *getpstr(void *d, short str_id, int x)
{
	long			z;
	UInt8			*s = NULL; /* not necessary considering unset->zero length */

	_scan_string(s,z);
	BlockMoveData(s,1+(UInt8*)d,z);
	*(UInt8*)d = z;

	return d;
}

int getcstr(char *d, short str_id, int x)
{
	long			z;
	UInt8			*s = NULL;

	_scan_string(s,z);
	BlockMoveData(s,d,z);
	d[z] = 0;

	return z;
}

int countstrlist(short id)
{
	Handle			h;
	UInt16			i;

	SetResLoad(0); /* must the first time */
	h = GetResource('STR#',id);
	LMSetResLoad(1);
	if (h==(Handle)NULL)
		return 0;
	ReadPartialResource(h,0,&i,2);
	assert(!ResError());

	return i;
}
#else /*defined(TARGET_OS_MAC)*/
UInt8 *getpstr(void *buf, short id, int x)
{
	int				length = getcstr((UInt8*)buf,id,x);

	memmove(1+(UInt8*)buf,buf,length);
	*(UInt8*)buf = length;
	return buf;
}
#endif
